from collections.abc import Sequence
from typing import Any

from django.http import HttpRequest, JsonResponse
from django_stubs_ext import StrOrPromise
from rest_framework.renderers import BaseRenderer
from rest_framework.request import Request
from typing_extensions import TypeAlias

def _get_error_details(data: Any, default_code: str | None = ...) -> Any: ...
def _get_codes(detail: Any) -> Any: ...
def _get_full_details(detail: Any) -> Any: ...

class ErrorDetail(str):
    code: str | None
    def __new__(cls, string: str, code: str | None = ...): ...

_Detail: TypeAlias = StrOrPromise | list[Any] | dict[str, Any]

class APIException(Exception):
    status_code: int
    default_detail: _Detail
    default_code: str

    detail: _Detail
    def __init__(self, detail: _Detail | None = ..., code: str | None = ...) -> None: ...
    def get_codes(self) -> Any: ...
    def get_full_details(self) -> Any: ...

class ValidationError(APIException): ...
class ParseError(APIException): ...
class AuthenticationFailed(APIException): ...
class NotAuthenticated(APIException): ...
class PermissionDenied(APIException): ...
class NotFound(APIException): ...

class MethodNotAllowed(APIException):
    def __init__(self, method: str, detail: _Detail | None = ..., code: str | None = ...) -> None: ...

class NotAcceptable(APIException):
    available_renderers: Sequence[BaseRenderer] | None
    def __init__(
        self,
        detail: _Detail | None = ...,
        code: str | None = ...,
        available_renderers: Sequence[BaseRenderer] | None = ...,
    ) -> None: ...

class UnsupportedMediaType(APIException):
    def __init__(self, media_type: str, detail: _Detail | None = ..., code: str | None = ...) -> None: ...

class Throttled(APIException):
    extra_detail_singular: str
    extra_detail_plural: str
    def __init__(self, wait: float | None = ..., detail: _Detail | None = ..., code: str | None = ...): ...

def server_error(request: HttpRequest | Request, *args: Any, **kwargs: Any) -> JsonResponse: ...
def bad_request(request: HttpRequest | Request, exception: Exception, *args: Any, **kwargs: Any) -> JsonResponse: ...



import sys
import types
from opcode import (
    EXTENDED_ARG as EXTENDED_ARG,
    HAVE_ARGUMENT as HAVE_ARGUMENT,
    cmp_op as cmp_op,
    hascompare as hascompare,
    hasconst as hasconst,
    hasfree as hasfree,
    hasjabs as hasjabs,
    hasjrel as hasjrel,
    haslocal as haslocal,
    hasname as hasname,
    opmap as opmap,
    opname as opname,
)
from typing import IO, Any, Callable, Dict, Iterator, List, NamedTuple, Optional, Tuple, Union

if sys.version_info >= (3, 4):
    from opcode import stack_effect as stack_effect

if sys.version_info >= (3, 6):
    from opcode import hasnargs as hasnargs

# Strictly this should not have to include Callable, but mypy doesn't use FunctionType
# for functions (python/mypy#3171)
_have_code = Union[types.MethodType, types.FunctionType, types.CodeType, type, Callable[..., Any]]
_have_code_or_string = Union[_have_code, str, bytes]

if sys.version_info >= (3, 4):
    class Instruction(NamedTuple):
        opname: str
        opcode: int
        arg: Optional[int]
        argval: Any
        argrepr: str
        offset: int
        starts_line: Optional[int]
        is_jump_target: bool
    class Bytecode:
        codeobj: types.CodeType
        first_line: int
        def __init__(
            self, x: _have_code_or_string, *, first_line: Optional[int] = ..., current_offset: Optional[int] = ...
        ) -> None: ...
        def __iter__(self) -> Iterator[Instruction]: ...
        def __repr__(self) -> str: ...
        def info(self) -> str: ...
        def dis(self) -> str: ...
        @classmethod
        def from_traceback(cls, tb: types.TracebackType) -> Bytecode: ...

COMPILER_FLAG_NAMES: Dict[int, str]

def findlabels(code: _have_code) -> List[int]: ...
def findlinestarts(code: _have_code) -> Iterator[Tuple[int, int]]: ...

if sys.version_info >= (3, 0):
    def pretty_flags(flags: int) -> str: ...
    def code_info(x: _have_code_or_string) -> str: ...

if sys.version_info >= (3, 7):
    def dis(x: Optional[_have_code_or_string] = ..., *, file: Optional[IO[str]] = ..., depth: Optional[int] = ...) -> None: ...

elif sys.version_info >= (3, 4):
    def dis(x: Optional[_have_code_or_string] = ..., *, file: Optional[IO[str]] = ...) -> None: ...

else:
    def dis(x: _have_code_or_string = ...) -> None: ...

if sys.version_info >= (3, 4):
    def distb(tb: Optional[types.TracebackType] = ..., *, file: Optional[IO[str]] = ...) -> None: ...
    def disassemble(co: _have_code, lasti: int = ..., *, file: Optional[IO[str]] = ...) -> None: ...
    def disco(co: _have_code, lasti: int = ..., *, file: Optional[IO[str]] = ...) -> None: ...
    def show_code(co: _have_code, *, file: Optional[IO[str]] = ...) -> None: ...
    def get_instructions(x: _have_code, *, first_line: Optional[int] = ...) -> Iterator[Instruction]: ...

else:
    def distb(tb: types.TracebackType = ...) -> None: ...
    def disassemble(co: _have_code, lasti: int = ...) -> None: ...
    def disco(co: _have_code, lasti: int = ...) -> None: ...
    if sys.version_info >= (3, 0):
        def show_code(co: _have_code) -> None: ...



from typing import Any

from django.http.request import HttpRequest
from django.http.response import HttpResponseBase, HttpResponsePermanentRedirect
from django.utils.deprecation import MiddlewareMixin

class CommonMiddleware(MiddlewareMixin):
    response_redirect_class: Any
    def process_request(self, request: HttpRequest) -> HttpResponsePermanentRedirect | None: ...
    def should_redirect_with_slash(self, request: HttpRequest) -> bool: ...
    def get_full_path_with_slash(self, request: HttpRequest) -> str: ...
    def process_response(self, request: HttpRequest, response: HttpResponseBase) -> HttpResponseBase: ...

class BrokenLinkEmailsMiddleware(MiddlewareMixin):
    def process_response(self, request: HttpRequest, response: HttpResponseBase) -> HttpResponseBase: ...
    def is_internal_request(self, domain: str, referer: str) -> bool: ...
    def is_ignorable_request(self, request: HttpRequest, uri: str, domain: str, referer: str) -> bool: ...



from typing import Any

COMMON_P: str
WORDS: Any
COMMON_WORDS: Any

def sentence() -> str: ...
def paragraph() -> str: ...
def paragraphs(count: int, common: bool = ...) -> list[str]: ...
def words(count: int, common: bool = ...) -> str: ...



from typing import Optional, OrderedDict, Type

from django.core.files.base import ContentFile
from django.db.models import Model
from rest_framework import serializers


class FileModelSerializer(serializers.ModelSerializer):
    """
    A serializer that allows giving a filename to a file that was uploaded
    as an base64 string.
    """

    class Meta:
        model: Type[Model]
        file_key: Optional[str] = None
        """ Key that holds the file """

        file_name_key: Optional[str] = None
        """ Key that holds the file name """

    def validate(self, attrs: OrderedDict):
        """Assign file name to file validating input data"""

        if not self.Meta.file_key or not self.Meta.file_name_key:
            raise Exception(
                "For a FileModelSerializer you have to define a file_key and a file_name_key"
            )

        validated_data = super().validate(attrs)

        file_name_key = self.Meta.file_name_key
        file_key = self.Meta.file_key

        if hasattr(validated_data, file_key) and hasattr(validated_data, file_name_key):
            file_name = validated_data[file_name_key]
            file = validated_data[file_key]

            if file_name and file:
                if isinstance(file, ContentFile):
                    file.name = file_name
                    validated_data[file_key] = file

            del validated_data[file_name_key]

        return validated_data



from collections.abc import Mapping, Sequence
from typing import Any, Optional, Protocol  # noqa: Y037  # https://github.com/python/mypy/issues/12211

from django.db.backends.base.schema import BaseDatabaseSchemaEditor
from django.db.migrations.state import StateApps
from django.utils.datastructures import _ListOrTuple
from typing_extensions import Literal

from .base import Operation

class SeparateDatabaseAndState(Operation):
    database_operations: Sequence[Operation]
    state_operations: Sequence[Operation]
    def __init__(
        self, database_operations: Sequence[Operation] = ..., state_operations: Sequence[Operation] = ...
    ) -> None: ...

class RunSQL(Operation):
    noop: Literal[""]
    sql: str | _ListOrTuple[str | tuple[str, dict[str, Any] | Optional[_ListOrTuple[str]]]]
    reverse_sql: str | None | _ListOrTuple[str | tuple[str, dict[str, Any] | Optional[_ListOrTuple[str]]]]
    state_operations: Sequence[Operation]
    hints: Mapping[str, Any]
    def __init__(
        self,
        sql: str | _ListOrTuple[str | tuple[str, dict[str, Any] | Optional[_ListOrTuple[str]]]],
        reverse_sql: str | None | _ListOrTuple[str | tuple[str, dict[str, Any] | Optional[_ListOrTuple[str]]]] = ...,
        state_operations: Sequence[Operation] = ...,
        hints: Mapping[str, Any] | None = ...,
        elidable: bool = ...,
    ) -> None: ...

class _CodeCallable(Protocol):
    def __call__(self, __state_apps: StateApps, __schema_editor: BaseDatabaseSchemaEditor) -> None: ...

class RunPython(Operation):
    code: _CodeCallable
    reverse_code: _CodeCallable | None
    hints: Mapping[str, Any]
    def __init__(
        self,
        code: _CodeCallable,
        reverse_code: _CodeCallable | None = ...,
        atomic: bool | None = ...,
        hints: Mapping[str, Any] | None = ...,
        elidable: bool = ...,
    ) -> None: ...
    @staticmethod
    def noop(apps: StateApps, schema_editor: BaseDatabaseSchemaEditor) -> None: ...



from typing import Any

from django.core.management.base import BaseCommand

class Command(BaseCommand):
    output_transaction: bool
    def execute(self, *args: Any, **options: Any) -> str | None: ...



from typing import Any

from kombu.connection import Connection

class Transport: ...

class Channel:
    def __init__(self, connection: Connection, **kwargs: Any) -> None: ...
    def close(self) -> None: ...

StdChannel = Channel



from collections.abc import Callable
from hmac import HMAC

using_sysrandom: bool
RANDOM_STRING_CHARS: str

def salted_hmac(
    key_salt: bytes | str, value: bytes | str, secret: bytes | str | None = ..., *, algorithm: str = ...
) -> HMAC: ...
def get_random_string(length: int = ..., allowed_chars: str = ...) -> str: ...
def constant_time_compare(val1: bytes | str, val2: bytes | str) -> bool: ...
def pbkdf2(
    password: bytes | str,
    salt: bytes | str,
    iterations: int,
    dklen: int = ...,
    digest: Callable | None = ...,
) -> bytes: ...



from collections.abc import Sequence
from typing import Any

from django.apps.config import AppConfig
from django.core.checks.messages import Warning

W003: Warning
W016: Warning

def check_csrf_middleware(app_configs: Sequence[AppConfig] | None, **kwargs: Any) -> Sequence[Warning]: ...
def check_csrf_cookie_secure(app_configs: Sequence[AppConfig] | None, **kwargs: Any) -> Sequence[Warning]: ...



from typing import List, Union

from django.db.models import Model, base
from pydantic import BaseModel


class SeedSet(BaseModel):
    models: List[Union[Model, base.ModelBase]]
    seed_name: str

    class Config:
        arbitrary_types_allowed = True



from collections.abc import Iterable, Mapping, Sequence
from json import JSONEncoder
from typing import Any

from django import forms
from rest_framework.request import Request
from rest_framework.response import Response
from rest_framework.serializers import BaseSerializer
from rest_framework.utils.field_mapping import ClassLookupDict
from rest_framework.views import APIView

def zero_as_none(value: Any) -> Any | None: ...

class BaseRenderer:
    media_type: str
    format: str
    charset: str | None
    render_style: str
    def render(
        self, data: Any, accepted_media_type: str | None = ..., renderer_context: Mapping[str, Any] | None = ...
    ) -> Any: ...

class JSONRenderer(BaseRenderer):
    encoder_class: type[JSONEncoder]
    ensure_ascii: bool
    compact: bool
    strict: bool
    def get_indent(self, accepted_media_type: str, renderer_context: Mapping[str, Any]) -> int | None: ...

class TemplateHTMLRenderer(BaseRenderer):
    template_name: str | None
    exception_template_names: Sequence[str]
    def resolve_template(self, template_names: Iterable[str]) -> Any: ...
    def get_template_context(self, data: Any, renderer_context: Mapping[str, Any]): ...
    def get_template_names(self, response: Response, view: APIView) -> list[str]: ...
    def get_exception_template(self, response: Response) -> Any: ...

class StaticHTMLRenderer(TemplateHTMLRenderer): ...

class HTMLFormRenderer(BaseRenderer):
    template_pack: str
    base_template: str

    default_style: ClassLookupDict
    def render_field(self, field, parent_style: Mapping[str, Any]) -> str: ...

class BrowsableAPIRenderer(BaseRenderer):
    """
    HTML renderer used to self-document the API.
    """

    template: str
    filter_template: str
    code_style: str
    form_renderer_class: type[BaseRenderer]
    def get_default_renderer(self, view: APIView) -> BaseRenderer: ...
    def get_content(
        self, renderer: BaseRenderer, data: Any, accepted_media_type: str, renderer_context: Mapping[str, Any]
    ) -> str: ...
    def show_form_for_method(self, view: APIView, method: str, request: Request, obj: Any) -> bool: ...
    def _get_serializer(
        self, serializer_class: type[BaseSerializer], view_instance: APIView, request: Request, *args, **kwargs
    ) -> BaseSerializer: ...
    def get_rendered_html_form(self, data: Any, view: APIView, method: str, request: Request) -> Any: ...
    def render_form_for_serializer(self, serializer: BaseSerializer) -> Any: ...
    def get_raw_data_form(self, data: Any, view: APIView, method: str, request: Request) -> forms.Form | None: ...
    def get_name(self, view: APIView) -> str: ...
    def get_description(self, view: APIView, status_code: int) -> str: ...
    def get_breadcrumbs(self, request: Request) -> list[tuple[str, str]]: ...
    def get_extra_actions(self, view: APIView) -> dict[str, str] | None: ...
    def get_filter_form(self, data: Any, view: APIView, request: Request) -> Any | None: ...
    def get_context(
        self, data: Any, accepted_media_type: str | None, renderer_context: Mapping[str, Any]
    ) -> dict[str, Any]: ...

class AdminRenderer(BrowsableAPIRenderer):
    def get_result_url(self, result: Mapping[str, Any], view: APIView) -> str | None: ...

class DocumentationRenderer(BaseRenderer):
    template: str
    error_template: str
    code_style: str
    languages: Sequence[str]
    def get_context(self, data: Any, request: Request) -> dict[str, Any]: ...

class SchemaJSRenderer(BaseRenderer):
    template: str

class MultiPartRenderer(BaseRenderer):
    BOUNDARY: str

class CoreJSONRenderer(BaseRenderer): ...

class _BaseOpenAPIRenderer:
    media_type: str
    charset: Any
    format: str
    def __init__(self) -> None: ...
    def render(self, data: Any, media_type: Any | None = ..., renderer_context: Any | None = ...): ...
    def get_schema(self, instance: Any) -> dict[str, Any]: ...
    def get_parameters(self, link) -> dict[str, Any]: ...
    def get_operation(self, link, name, tag) -> dict[str, Any]: ...
    def get_paths(self, document) -> dict[str, Any]: ...
    def get_structure(self, data: Any) -> dict[str, Any]: ...

class JSONOpenAPIRenderer(_BaseOpenAPIRenderer): ...
class OpenAPIRenderer(_BaseOpenAPIRenderer): ...
class CoreAPIOpenAPIRenderer(_BaseOpenAPIRenderer): ...
class CoreAPIJSONOpenAPIRenderer(_BaseOpenAPIRenderer): ...



import sys
from _typeshed import ReadableBuffer
from typing_extensions import TypeAlias

# Many functions in binascii accept buffer objects
# or ASCII-only strings.
_AsciiBuffer: TypeAlias = str | ReadableBuffer

def a2b_uu(__data: _AsciiBuffer) -> bytes: ...
def b2a_uu(__data: ReadableBuffer, *, backtick: bool = False) -> bytes: ...

if sys.version_info >= (3, 11):
    def a2b_base64(__data: _AsciiBuffer, *, strict_mode: bool = False) -> bytes: ...

else:
    def a2b_base64(__data: _AsciiBuffer) -> bytes: ...

def b2a_base64(__data: ReadableBuffer, *, newline: bool = True) -> bytes: ...
def a2b_qp(data: _AsciiBuffer, header: bool = False) -> bytes: ...
def b2a_qp(data: ReadableBuffer, quotetabs: bool = False, istext: bool = True, header: bool = False) -> bytes: ...

if sys.version_info < (3, 11):
    def a2b_hqx(__data: _AsciiBuffer) -> bytes: ...
    def rledecode_hqx(__data: ReadableBuffer) -> bytes: ...
    def rlecode_hqx(__data: ReadableBuffer) -> bytes: ...
    def b2a_hqx(__data: ReadableBuffer) -> bytes: ...

def crc_hqx(__data: ReadableBuffer, __crc: int) -> int: ...
def crc32(__data: ReadableBuffer, __crc: int = 0) -> int: ...

if sys.version_info >= (3, 8):
    # sep must be str or bytes, not bytearray or any other buffer
    def b2a_hex(data: ReadableBuffer, sep: str | bytes = ..., bytes_per_sep: int = ...) -> bytes: ...
    def hexlify(data: ReadableBuffer, sep: str | bytes = ..., bytes_per_sep: int = ...) -> bytes: ...

else:
    def b2a_hex(__data: ReadableBuffer) -> bytes: ...
    def hexlify(__data: ReadableBuffer) -> bytes: ...

def a2b_hex(__hexstr: _AsciiBuffer) -> bytes: ...
def unhexlify(__hexstr: _AsciiBuffer) -> bytes: ...

class Error(ValueError): ...
class Incomplete(Exception): ...



import logging
from typing import TextIO

from . import connectionpool, filepost, poolmanager, response
from .util import request as _request, retry, timeout, url

__author__: str
__license__: str
__version__: str

HTTPConnectionPool = connectionpool.HTTPConnectionPool
HTTPSConnectionPool = connectionpool.HTTPSConnectionPool
connection_from_url = connectionpool.connection_from_url
encode_multipart_formdata = filepost.encode_multipart_formdata
PoolManager = poolmanager.PoolManager
ProxyManager = poolmanager.ProxyManager
proxy_from_url = poolmanager.proxy_from_url
HTTPResponse = response.HTTPResponse
make_headers = _request.make_headers
get_host = url.get_host
Timeout = timeout.Timeout
Retry = retry.Retry

class NullHandler(logging.Handler):
    def emit(self, record): ...

def add_stderr_logger(level: int = 10) -> logging.StreamHandler[TextIO]: ...
def disable_warnings(category: type[Warning] = ...) -> None: ...



GDAL_PIXEL_TYPES: dict[int, str]
GDAL_INTEGER_TYPES: list[int]
GDAL_TO_CTYPES: type
GDAL_RESAMPLE_ALGORITHMS: dict[str, int]
GDAL_COLOR_TYPES: dict[int, str]
VSI_FILESYSTEM_BASE_PATH: str
VSI_TAKE_BUFFER_OWNERSHIP: bool
VSI_DELETE_BUFFER_ON_READ: bool



from typing import Any

from django.contrib.admin import ModelAdmin
from django.contrib.gis.forms import BaseGeometryWidget

spherical_mercator_srid: int

class GISModelAdmin(ModelAdmin):
    gis_widget: BaseGeometryWidget
    gis_widget_kwargs: dict[str, Any]

class GeoModelAdmin(ModelAdmin):
    default_lon: int
    default_lat: int
    default_zoom: int
    display_wkt: bool
    display_srid: bool
    extra_js: Any
    num_zoom: int
    max_zoom: bool
    min_zoom: bool
    units: str | bool
    max_resolution: str | bool
    max_extent: str | bool
    modifiable: bool
    mouse_position: bool
    scale_text: bool
    layerswitcher: bool
    scrollable: bool
    map_width: int
    map_height: int
    map_srid: int
    map_template: str
    openlayers_url: str
    point_zoom: Any
    wms_url: str
    wms_layer: str
    wms_name: str
    wms_options: Any
    debug: bool
    widget: Any
    @property
    def media(self) -> Any: ...
    def get_map_widget(self, db_field: Any) -> Any: ...

class OSMGeoAdmin(GeoModelAdmin):
    map_template: str
    num_zoom: int
    map_srid: Any
    max_extent: str
    max_resolution: str
    point_zoom: int
    units: str



from typing import Any, ClassVar
from typing_extensions import Literal

from .ImageFile import ImageFile

xbm_head: Any

class XbmImageFile(ImageFile):
    format: ClassVar[Literal["XBM"]]
    format_description: ClassVar[str]



import sys
from collections.abc import Callable
from typing import TypeVar
from typing_extensions import Concatenate, ParamSpec

if sys.platform != "win32":
    from _curses import *
    from _curses import _CursesWindow as _CursesWindow

    _T = TypeVar("_T")
    _P = ParamSpec("_P")

    # available after calling `curses.initscr()`
    LINES: int
    COLS: int

    # available after calling `curses.start_color()`
    COLORS: int
    COLOR_PAIRS: int

    def wrapper(__func: Callable[Concatenate[_CursesWindow, _P], _T], *arg: _P.args, **kwds: _P.kwargs) -> _T: ...



import sys
from _typeshed import ReadableBuffer, StrPath, SupportsRead, _T_co
from collections.abc import Iterable
from typing import Any, NoReturn, Protocol
from xml.sax.handler import ContentHandler as ContentHandler, ErrorHandler as ErrorHandler
from xml.sax.xmlreader import Locator, XMLReader

class _SupportsReadClose(SupportsRead[_T_co], Protocol[_T_co]):
    def close(self) -> None: ...

class SAXException(Exception):
    def __init__(self, msg: str, exception: Exception | None = None) -> None: ...
    def getMessage(self) -> str: ...
    def getException(self) -> Exception: ...
    def __getitem__(self, ix: Any) -> NoReturn: ...

class SAXParseException(SAXException):
    def __init__(self, msg: str, exception: Exception | None, locator: Locator) -> None: ...
    def getColumnNumber(self) -> int: ...
    def getLineNumber(self) -> int: ...
    def getPublicId(self): ...
    def getSystemId(self): ...

class SAXNotRecognizedException(SAXException): ...
class SAXNotSupportedException(SAXException): ...
class SAXReaderNotAvailable(SAXNotSupportedException): ...

default_parser_list: list[str]

if sys.version_info >= (3, 8):
    def make_parser(parser_list: Iterable[str] = ()) -> XMLReader: ...
    def parse(
        source: StrPath | _SupportsReadClose[bytes] | _SupportsReadClose[str],
        handler: ContentHandler,
        errorHandler: ErrorHandler = ...,
    ) -> None: ...

else:
    def make_parser(parser_list: list[str] = []) -> XMLReader: ...
    def parse(
        source: str | _SupportsReadClose[bytes] | _SupportsReadClose[str],
        handler: ContentHandler,
        errorHandler: ErrorHandler = ...,
    ) -> None: ...

def parseString(string: ReadableBuffer | str, handler: ContentHandler, errorHandler: ErrorHandler | None = ...) -> None: ...
def _create_parser(parser_name: str) -> XMLReader: ...



from decimal import Decimal
from typing import Any

from _typeshed import Self
from typing_extensions import TypeAlias

_NUMERIC_TYPES: TypeAlias = int | float | Decimal

class MeasureBase:
    STANDARD_UNIT: str | None
    UNITS: dict[str, float]
    ALIAS: dict[str, str]
    LALIAS: dict[str, str]
    def __init__(self, default_unit: str | None = ..., **kwargs: Any) -> None: ...
    standard: Any
    def __getattr__(self, name: str) -> float: ...
    def __eq__(self, other: object) -> bool: ...
    def __lt__(self: Self, other: Self) -> bool: ...
    def __add__(self: Self, other: Self) -> Self: ...
    def __iadd__(self: Self, other: Self) -> Self: ...
    def __sub__(self: Self, other: Self) -> Self: ...
    def __isub__(self: Self, other: Self) -> Self: ...
    def __mul__(self, other: _NUMERIC_TYPES) -> Any: ...
    def __imul__(self: Self, other: _NUMERIC_TYPES) -> Self: ...
    def __rmul__(self: Self, other: Any) -> Self: ...
    def __truediv__(self: Self, other: _NUMERIC_TYPES) -> Self: ...
    def __itruediv__(self: Self, other: _NUMERIC_TYPES) -> Self: ...
    def __bool__(self) -> bool: ...
    def default_units(self, kwargs: dict[str, Any]) -> tuple[float, str]: ...
    @classmethod
    def unit_attname(cls, unit_str: str) -> str: ...

class Distance(MeasureBase):
    STANDARD_UNIT: str

class Area(MeasureBase):
    STANDARD_UNIT: str

D: TypeAlias = Distance

A: TypeAlias = Area



from typing import Any, ClassVar
from typing_extensions import Final, Literal

from .ImageFile import ImageFile, PyDecoder

BIT2MODE: Any

class BmpImageFile(ImageFile):
    RAW: Final = 0
    RLE8: Final = 1
    RLE4: Final = 2
    BITFIELDS: Final = 3
    JPEG: Final = 4
    PNG: Final = 5
    format_description: ClassVar[str]
    format: ClassVar[Literal["BMP", "DIB", "CUR"]]
    COMPRESSIONS: Any

class BmpRleDecoder(PyDecoder):
    def decode(self, buffer): ...

class DibImageFile(BmpImageFile):
    format: ClassVar[Literal["DIB"]]

SAVE: Any



import codecs
from typing import Text, Tuple

class IncrementalEncoder(codecs.IncrementalEncoder):
    def encode(self, input: Text, final: bool = ...) -> bytes: ...

class IncrementalDecoder(codecs.BufferedIncrementalDecoder):
    def _buffer_decode(self, input: bytes, errors: str, final: bool) -> Tuple[Text, int]: ...

class StreamWriter(codecs.StreamWriter): ...
class StreamReader(codecs.StreamReader): ...

def getregentry() -> codecs.CodecInfo: ...
def encode(input: Text, errors: Text = ...) -> bytes: ...
def decode(input: bytes, errors: Text = ...) -> Text: ...



from django.contrib.auth.backends import AllowAllUsersModelBackend
from django.contrib.auth.models import Permission


class AllowAllUsersModelBackendIncludingInactive(AllowAllUsersModelBackend):
    """
    Permission backend that grants the same permissions fo active and inactive user.

    Default behavior is that inactive user don't get any permissions at all.
    """

    def _get_permissions(self, user_obj, obj, from_name):
        """
        Return the permissions of `user_obj` from `from_name`. `from_name` can
        be either "group" or "user" to return permissions from
        `_get_group_permissions` or `_get_user_permissions` respectively.
        """
        if user_obj.is_anonymous or obj is not None:
            return set()

        perm_cache_name = "_%s_perm_cache" % from_name
        if not hasattr(user_obj, perm_cache_name):
            if user_obj.is_superuser:
                perms = Permission.objects.all()
            else:
                perms = getattr(self, "_get_%s_permissions" % from_name)(user_obj)
            perms = perms.values_list("content_type__app_label", "codename").order_by()
            setattr(
                user_obj, perm_cache_name, {"%s.%s" % (ct, name) for ct, name in perms}
            )
        return getattr(user_obj, perm_cache_name)

    def get_all_permissions(self, user_obj, obj=None):
        if user_obj.is_anonymous or obj is not None:
            return set()

        if not hasattr(user_obj, "_perm_cache"):
            user_obj._perm_cache = {
                *self.get_user_permissions(user_obj, obj=obj),
                *self.get_group_permissions(user_obj, obj=obj),
            }

        return user_obj._perm_cache

    def has_perm(self, user_obj, perm, obj=None):
        return perm in self.get_all_permissions(user_obj, obj=obj)

    def has_module_perms(self, user_obj, app_label):
        """
        Return True if user_obj has any permissions in the given app_label.
        """
        return any(
            perm[: perm.index(".")] == app_label
            for perm in self.get_all_permissions(user_obj)
        )

    def with_perm(self, perm, is_active=None, include_superusers=True, obj=None):
        return super().with_perm(perm, is_active, include_superusers, obj)



from collections import OrderedDict
from collections.abc import Iterator, MutableMapping
from typing import Any

from rest_framework.exceptions import ErrorDetail
from rest_framework.fields import Field
from rest_framework.serializers import BaseSerializer

class ReturnDict(OrderedDict):
    serializer: BaseSerializer
    def __init__(self, serializer: BaseSerializer = ..., *args, **kwargs): ...
    def copy(self) -> ReturnDict: ...
    def __reduce__(self) -> tuple[dict, tuple[dict]]: ...

class ReturnList(list):
    serializer: BaseSerializer
    def __init__(self, serializer: BaseSerializer = ..., *args, **kwargs): ...
    def __reduce__(self) -> tuple[dict, tuple[dict]]: ...

class BoundField:
    """
    A field object that also includes `.value` and `.error` properties.
    Returned when iterating over a serializer instance,
    providing an API similar to Django forms and form fields.
    """

    value: Any
    fields: dict[str, Field]
    errors: list[ErrorDetail]
    def __init__(self, field: Field, value: Any, errors: list[ErrorDetail], prefix: str = ...): ...
    def __getattr__(self, attr_name: str) -> Any: ...
    def as_form_field(self) -> BoundField: ...

class JSONBoundField(BoundField): ...

class NestedBoundField(BoundField):
    def __iter__(self) -> Iterator[str]: ...
    def __getitem__(self, key: str) -> BoundField | NestedBoundField: ...

class BindingDict(MutableMapping[str, Field]):
    serializer: BaseSerializer
    fields: OrderedDict[str, Field]
    def __init__(self, serializer: BaseSerializer): ...
    def __setitem__(self, key: str, field: Field) -> None: ...
    def __getitem__(self, key: str) -> Field: ...
    def __delitem__(self, key: str) -> None: ...
    def __iter__(self) -> Iterator[Field]: ...  # type: ignore[override]
    def __len__(self) -> int: ...



from typing import Any

class PostGISAdapter:
    is_geometry: Any
    ewkb: Any
    srid: Any
    geography: Any
    def __init__(self, obj: Any, geography: bool = ...) -> None: ...
    def __conform__(self, proto: Any) -> Any: ...
    def __eq__(self, other: object) -> bool: ...
    def __hash__(self) -> int: ...
    def prepare(self, conn: Any) -> None: ...
    def getquoted(self) -> Any: ...



from collections.abc import Awaitable, Callable
from typing import Any

from django.http.request import HttpRequest
from django.http.response import HttpResponse, HttpResponseBase
from django.urls.resolvers import URLResolver

def convert_exception_to_response(
    get_response: Callable[[HttpRequest], HttpResponseBase | Awaitable[HttpResponseBase]]
) -> Callable[[HttpRequest], HttpResponseBase | Awaitable[HttpResponseBase]]: ...
def response_for_exception(request: HttpRequest, exc: Exception) -> HttpResponse: ...
def get_exception_response(
    request: HttpRequest, resolver: URLResolver, status_code: int, exception: Exception
) -> HttpResponse: ...
def handle_uncaught_exception(request: HttpRequest, resolver: URLResolver, exc_info: Any) -> HttpResponse: ...



import sys
from typing import Any, Callable, ClassVar, Generic, Iterator, Mapping, Optional, TypeVar, Union, overload

if sys.version_info >= (3, 9):
    from types import GenericAlias

_T = TypeVar("_T")
_D = TypeVar("_D")

class ContextVar(Generic[_T]):
    def __init__(self, name: str, *, default: _T = ...) -> None: ...
    @property
    def name(self) -> str: ...
    @overload
    def get(self) -> _T: ...
    @overload
    def get(self, default: Union[_D, _T]) -> Union[_D, _T]: ...
    def set(self, value: _T) -> Token[_T]: ...
    def reset(self, token: Token[_T]) -> None: ...
    if sys.version_info >= (3, 9):
        def __class_getitem__(cls, item: Any) -> GenericAlias: ...

class Token(Generic[_T]):
    @property
    def var(self) -> ContextVar[_T]: ...
    @property
    def old_value(self) -> Any: ...  # returns either _T or MISSING, but that's hard to express
    MISSING: ClassVar[object]
    if sys.version_info >= (3, 9):
        def __class_getitem__(cls, item: Any) -> GenericAlias: ...

def copy_context() -> Context: ...

# It doesn't make sense to make this generic, because for most Contexts each ContextVar will have
# a different value.
class Context(Mapping[ContextVar[Any], Any]):
    def __init__(self) -> None: ...
    @overload
    def get(self, __key: ContextVar[Any]) -> Optional[Any]: ...
    @overload
    def get(self, __key: ContextVar[Any], __default: Optional[Any]) -> Any: ...
    def run(self, callable: Callable[..., _T], *args: Any, **kwargs: Any) -> _T: ...
    def copy(self) -> Context: ...
    def __getitem__(self, key: ContextVar[Any]) -> Any: ...
    def __iter__(self) -> Iterator[ContextVar[Any]]: ...
    def __len__(self) -> int: ...



import time
from contextlib import contextmanager

from django.core.cache import cache

LOCK_EXPIRE = 60 * 10  # Lock expires in 10 minutes


@contextmanager
def memcache_lock(lock_id, oid, lock_expiration_after_minutes: int = LOCK_EXPIRE):
    """
    Preventing duplicate tasks via usage of the caching backend
    """

    timeout_at = time.monotonic() + lock_expiration_after_minutes - 3

    # cache.add fails if the key already exists
    status = cache.add(lock_id, oid, lock_expiration_after_minutes)

    try:
        yield status
    finally:
        # memcache delete is very slow, but we have to use it to take
        # advantage of using add() for atomic locking
        if time.monotonic() < timeout_at and status:
            # don't release the lock if we exceeded the timeout
            # to lessen the chance of releasing an expired lock
            # owned by someone else
            # also don't release the lock if we didn't acquire it
            cache.delete(lock_id)



import mimetools
import ssl
from typing import Any, Dict, Optional, Protocol

class HTTPMessage(mimetools.Message):
    def addcontinue(self, key: str, more: str) -> None: ...
    dict: Dict[str, str]
    def addheader(self, key: str, value: str) -> None: ...
    unixfrom: str
    headers: Any
    status: str
    seekable: bool
    def readheaders(self) -> None: ...

class HTTPResponse:
    fp: Any
    debuglevel: Any
    strict: Any
    msg: Any
    version: Any
    status: Any
    reason: Any
    chunked: Any
    chunk_left: Any
    length: Any
    will_close: Any
    def __init__(
        self, sock, debuglevel: int = ..., strict: int = ..., method: Optional[Any] = ..., buffering: bool = ...
    ) -> None: ...
    def begin(self): ...
    def close(self): ...
    def isclosed(self): ...
    def read(self, amt: Optional[Any] = ...): ...
    def fileno(self): ...
    def getheader(self, name, default: Optional[Any] = ...): ...
    def getheaders(self): ...

# This is an API stub only for HTTPConnection and HTTPSConnection, as used in
# urllib2.AbstractHTTPHandler.do_open, which takes either the class
# HTTPConnection or the class HTTPSConnection, *not* an instance of either
# class. do_open does not use all of the parameters of HTTPConnection.__init__
# or HTTPSConnection.__init__, so HTTPConnectionProtocol only implements the
# parameters that do_open does use.
class HTTPConnectionProtocol(Protocol):
    def __call__(self, host: str, timeout: int = ..., **http_con_args: Any) -> HTTPConnection: ...

class HTTPConnection:
    response_class: Any
    default_port: Any
    auto_open: Any
    debuglevel: Any
    strict: Any
    timeout: Any
    source_address: Any
    sock: Any
    host: str = ...
    port: int = ...
    def __init__(
        self, host, port: Optional[Any] = ..., strict: Optional[Any] = ..., timeout=..., source_address: Optional[Any] = ...
    ) -> None: ...
    def set_tunnel(self, host, port: Optional[Any] = ..., headers: Optional[Any] = ...): ...
    def set_debuglevel(self, level): ...
    def connect(self): ...
    def close(self): ...
    def send(self, data): ...
    def putrequest(self, method, url, skip_host: int = ..., skip_accept_encoding: int = ...): ...
    def putheader(self, header, *values): ...
    def endheaders(self, message_body: Optional[Any] = ...): ...
    def request(self, method, url, body: Optional[Any] = ..., headers=...): ...
    def getresponse(self, buffering: bool = ...): ...

class HTTP:
    debuglevel: Any
    def __init__(self, host: str = ..., port: Optional[Any] = ..., strict: Optional[Any] = ...) -> None: ...
    def connect(self, host: Optional[Any] = ..., port: Optional[Any] = ...): ...
    def getfile(self): ...
    file: Any
    headers: Any
    def getreply(self, buffering: bool = ...): ...
    def close(self): ...

class HTTPSConnection(HTTPConnection):
    default_port: Any
    key_file: Any
    cert_file: Any
    def __init__(
        self,
        host,
        port: Optional[Any] = ...,
        key_file: Optional[Any] = ...,
        cert_file: Optional[Any] = ...,
        strict: Optional[Any] = ...,
        timeout=...,
        source_address: Optional[Any] = ...,
        context: Optional[Any] = ...,
    ) -> None: ...
    sock: Any
    def connect(self): ...

class HTTPS(HTTP):
    key_file: Any
    cert_file: Any
    def __init__(
        self,
        host: str = ...,
        port: Optional[Any] = ...,
        key_file: Optional[Any] = ...,
        cert_file: Optional[Any] = ...,
        strict: Optional[Any] = ...,
        context: Optional[Any] = ...,
    ) -> None: ...

class HTTPException(Exception): ...
class NotConnected(HTTPException): ...
class InvalidURL(HTTPException): ...

class UnknownProtocol(HTTPException):
    args: Any
    version: Any
    def __init__(self, version) -> None: ...

class UnknownTransferEncoding(HTTPException): ...
class UnimplementedFileMode(HTTPException): ...

class IncompleteRead(HTTPException):
    args: Any
    partial: Any
    expected: Any
    def __init__(self, partial, expected: Optional[Any] = ...) -> None: ...

class ImproperConnectionState(HTTPException): ...
class CannotSendRequest(ImproperConnectionState): ...
class CannotSendHeader(ImproperConnectionState): ...
class ResponseNotReady(ImproperConnectionState): ...

class BadStatusLine(HTTPException):
    args: Any
    line: Any
    def __init__(self, line) -> None: ...

class LineTooLong(HTTPException):
    def __init__(self, line_type) -> None: ...

error: Any

class LineAndFileWrapper:
    def __init__(self, line, file) -> None: ...
    def __getattr__(self, attr): ...
    def read(self, amt: Optional[Any] = ...): ...
    def readline(self): ...
    def readlines(self, size: Optional[Any] = ...): ...

# Constants

responses: Dict[int, str]

HTTP_PORT: int
HTTPS_PORT: int

# status codes
# informational
CONTINUE: int
SWITCHING_PROTOCOLS: int
PROCESSING: int

# successful
OK: int
CREATED: int
ACCEPTED: int
NON_AUTHORITATIVE_INFORMATION: int
NO_CONTENT: int
RESET_CONTENT: int
PARTIAL_CONTENT: int
MULTI_STATUS: int
IM_USED: int

# redirection
MULTIPLE_CHOICES: int
MOVED_PERMANENTLY: int
FOUND: int
SEE_OTHER: int
NOT_MODIFIED: int
USE_PROXY: int
TEMPORARY_REDIRECT: int

# client error
BAD_REQUEST: int
UNAUTHORIZED: int
PAYMENT_REQUIRED: int
FORBIDDEN: int
NOT_FOUND: int
METHOD_NOT_ALLOWED: int
NOT_ACCEPTABLE: int
PROXY_AUTHENTICATION_REQUIRED: int
REQUEST_TIMEOUT: int
CONFLICT: int
GONE: int
LENGTH_REQUIRED: int
PRECONDITION_FAILED: int
REQUEST_ENTITY_TOO_LARGE: int
REQUEST_URI_TOO_LONG: int
UNSUPPORTED_MEDIA_TYPE: int
REQUESTED_RANGE_NOT_SATISFIABLE: int
EXPECTATION_FAILED: int
UNPROCESSABLE_ENTITY: int
LOCKED: int
FAILED_DEPENDENCY: int
UPGRADE_REQUIRED: int

# server error
INTERNAL_SERVER_ERROR: int
NOT_IMPLEMENTED: int
BAD_GATEWAY: int
SERVICE_UNAVAILABLE: int
GATEWAY_TIMEOUT: int
HTTP_VERSION_NOT_SUPPORTED: int
INSUFFICIENT_STORAGE: int
NOT_EXTENDED: int



import numbers
from datetime import datetime, timedelta
from typing import Callable, List, NamedTuple, Optional, Set, Tuple, Union

import ephem
from celery.app.base import Celery
from celery.utils.time import ffwd
from typing_extensions import Literal

class schedstate(NamedTuple):
    is_due: bool
    next: float

def cronfield(s: Optional[str]) -> str: ...

class ParseException(Exception): ...

class BaseSchedule:
    nowfunc: Callable[[], datetime]
    def __init__(
        self,
        nowfun: Optional[Callable[[], datetime]] = ...,
        app: Optional[Celery] = ...,
    ) -> None: ...
    def now(self) -> datetime: ...
    def remaining_estimate(self, last_run_at: datetime) -> timedelta: ...
    def is_due(self, last_run_at: datetime) -> schedstate: ...
    def maybe_make_aware(self, dt: datetime) -> datetime: ...
    @property
    def app(self) -> Celery: ...
    @app.setter
    def app(self, app: Celery) -> None: ...
    @property
    def tz(self) -> str: ...
    @property
    def utc_enabled(self) -> bool: ...
    def to_local(self, dt: datetime) -> datetime: ...
    def __eq__(self, other: object) -> bool: ...

class schedule(BaseSchedule):
    def __init__(
        self,
        run_every: Optional[Union[float, timedelta]] = ...,
        relative: bool = ...,
        nowfun: Optional[Callable[[], datetime]] = ...,
        app: Optional[Celery] = ...,
    ) -> None: ...
    @property
    def seconds(self) -> int: ...
    @property
    def human_seconds(self) -> str: ...

_ModuleLevelParseException = ParseException

class crontab_parser:
    ParseException: _ModuleLevelParseException
    def __init__(self, max_: int = ..., min_: int = ...) -> None: ...
    def parse(self, spec: str) -> Set[int]: ...

class crontab(BaseSchedule):
    def __init__(
        self,
        minute: Union[str, int, List[int]] = ...,
        hour: Union[str, int, List[int]] = ...,
        day_of_week: Union[str, int, List[int]] = ...,
        day_of_month: Union[str, int, List[int]] = ...,
        month_of_year: Union[str, int, List[int]] = ...,
        nowfun: Optional[Callable[[], datetime]] = ...,
        app: Optional[Celery] = ...,
    ) -> None: ...
    def remaining_delta(
        self, last_run_at: datetime, tz: Optional[str] = ..., ffwd: ffwd = ...
    ) -> Tuple[datetime, timedelta, datetime]: ...

def maybe_schedule(s: Union[numbers.Number, timedelta, BaseSchedule]) -> schedule: ...

_SolarEvent = Literal[
    "dawn_astronomical",
    "dawn_nautical",
    "dawn_civil",
    "sunrise",
    "solar_noon",
    "sunset",
    "dusk_civil",
    "dusk_nautical",
    "dusk_astronomical",
]

class solar(BaseSchedule):
    cal: ephem.Observer
    event: _SolarEvent
    lat: float
    lon: float
    def __init__(
        self,
        event: _SolarEvent,
        lat: float,
        lon: float,
        nowfun: Optional[Callable[[], datetime]] = ...,
        app: Optional[Celery] = ...,
    ) -> None: ...



from django.db import models


class NWONBaseModel(models.Model):
    """
    A abstraction over the Django base model.

    The models comes with enabled validation check on save.

    The models also has fields for keeping the created timestamp and an updated
    timestamp.
    """

    created = models.DateTimeField(
        auto_now_add=True,
        null=False,
        blank=True,
        help_text="The time when the model instance was created",
    )

    updated = models.DateTimeField(
        auto_now=True,
        null=False,
        blank=True,
        help_text="The last time the model instance was updated.",
    )

    def save(self, *args, **kwargs):
        self.full_clean()
        return super().save(*args, **kwargs)

    class Meta:
        abstract = True



from collections.abc import Mapping
from enum import IntEnum

class Base(IntEnum):
    InteropIndex: int
    ProcessingSoftware: int
    NewSubfileType: int
    SubfileType: int
    ImageWidth: int
    ImageLength: int
    BitsPerSample: int
    Compression: int
    PhotometricInterpretation: int
    Thresholding: int
    CellWidth: int
    CellLength: int
    FillOrder: int
    DocumentName: int
    ImageDescription: int
    Make: int
    Model: int
    StripOffsets: int
    Orientation: int
    SamplesPerPixel: int
    RowsPerStrip: int
    StripByteCounts: int
    MinSampleValue: int
    MaxSampleValue: int
    XResolution: int
    YResolution: int
    PlanarConfiguration: int
    PageName: int
    FreeOffsets: int
    FreeByteCounts: int
    GrayResponseUnit: int
    GrayResponseCurve: int
    T4Options: int
    T6Options: int
    ResolutionUnit: int
    PageNumber: int
    TransferFunction: int
    Software: int
    DateTime: int
    Artist: int
    HostComputer: int
    Predictor: int
    WhitePoint: int
    PrimaryChromaticities: int
    ColorMap: int
    HalftoneHints: int
    TileWidth: int
    TileLength: int
    TileOffsets: int
    TileByteCounts: int
    SubIFDs: int
    InkSet: int
    InkNames: int
    NumberOfInks: int
    DotRange: int
    TargetPrinter: int
    ExtraSamples: int
    SampleFormat: int
    SMinSampleValue: int
    SMaxSampleValue: int
    TransferRange: int
    ClipPath: int
    XClipPathUnits: int
    YClipPathUnits: int
    Indexed: int
    JPEGTables: int
    OPIProxy: int
    JPEGProc: int
    JpegIFOffset: int
    JpegIFByteCount: int
    JpegRestartInterval: int
    JpegLosslessPredictors: int
    JpegPointTransforms: int
    JpegQTables: int
    JpegDCTables: int
    JpegACTables: int
    YCbCrCoefficients: int
    YCbCrSubSampling: int
    YCbCrPositioning: int
    ReferenceBlackWhite: int
    XMLPacket: int
    RelatedImageFileFormat: int
    RelatedImageWidth: int
    RelatedImageLength: int
    Rating: int
    RatingPercent: int
    ImageID: int
    CFARepeatPatternDim: int
    BatteryLevel: int
    Copyright: int
    ExposureTime: int
    FNumber: int
    IPTCNAA: int
    ImageResources: int
    ExifOffset: int
    InterColorProfile: int
    ExposureProgram: int
    SpectralSensitivity: int
    GPSInfo: int
    ISOSpeedRatings: int
    OECF: int
    Interlace: int
    TimeZoneOffset: int
    SelfTimerMode: int
    SensitivityType: int
    StandardOutputSensitivity: int
    RecommendedExposureIndex: int
    ISOSpeed: int
    ISOSpeedLatitudeyyy: int
    ISOSpeedLatitudezzz: int
    ExifVersion: int
    DateTimeOriginal: int
    DateTimeDigitized: int
    OffsetTime: int
    OffsetTimeOriginal: int
    OffsetTimeDigitized: int
    ComponentsConfiguration: int
    CompressedBitsPerPixel: int
    ShutterSpeedValue: int
    ApertureValue: int
    BrightnessValue: int
    ExposureBiasValue: int
    MaxApertureValue: int
    SubjectDistance: int
    MeteringMode: int
    LightSource: int
    Flash: int
    FocalLength: int
    Noise: int
    ImageNumber: int
    SecurityClassification: int
    ImageHistory: int
    TIFFEPStandardID: int
    MakerNote: int
    UserComment: int
    SubsecTime: int
    SubsecTimeOriginal: int
    SubsecTimeDigitized: int
    AmbientTemperature: int
    Humidity: int
    Pressure: int
    WaterDepth: int
    Acceleration: int
    CameraElevationAngle: int
    XPTitle: int
    XPComment: int
    XPAuthor: int
    XPKeywords: int
    XPSubject: int
    FlashPixVersion: int
    ColorSpace: int
    ExifImageWidth: int
    ExifImageHeight: int
    RelatedSoundFile: int
    ExifInteroperabilityOffset: int
    FlashEnergy: int
    SpatialFrequencyResponse: int
    FocalPlaneXResolution: int
    FocalPlaneYResolution: int
    FocalPlaneResolutionUnit: int
    SubjectLocation: int
    ExposureIndex: int
    SensingMethod: int
    FileSource: int
    SceneType: int
    CFAPattern: int
    CustomRendered: int
    ExposureMode: int
    WhiteBalance: int
    DigitalZoomRatio: int
    FocalLengthIn35mmFilm: int
    SceneCaptureType: int
    GainControl: int
    Contrast: int
    Saturation: int
    Sharpness: int
    DeviceSettingDescription: int
    SubjectDistanceRange: int
    ImageUniqueID: int
    CameraOwnerName: int
    BodySerialNumber: int
    LensSpecification: int
    LensMake: int
    LensModel: int
    LensSerialNumber: int
    CompositeImage: int
    CompositeImageCount: int
    CompositeImageExposureTimes: int
    Gamma: int
    PrintImageMatching: int
    DNGVersion: int
    DNGBackwardVersion: int
    UniqueCameraModel: int
    LocalizedCameraModel: int
    CFAPlaneColor: int
    CFALayout: int
    LinearizationTable: int
    BlackLevelRepeatDim: int
    BlackLevel: int
    BlackLevelDeltaH: int
    BlackLevelDeltaV: int
    WhiteLevel: int
    DefaultScale: int
    DefaultCropOrigin: int
    DefaultCropSize: int
    ColorMatrix1: int
    ColorMatrix2: int
    CameraCalibration1: int
    CameraCalibration2: int
    ReductionMatrix1: int
    ReductionMatrix2: int
    AnalogBalance: int
    AsShotNeutral: int
    AsShotWhiteXY: int
    BaselineExposure: int
    BaselineNoise: int
    BaselineSharpness: int
    BayerGreenSplit: int
    LinearResponseLimit: int
    CameraSerialNumber: int
    LensInfo: int
    ChromaBlurRadius: int
    AntiAliasStrength: int
    ShadowScale: int
    DNGPrivateData: int
    MakerNoteSafety: int
    CalibrationIlluminant1: int
    CalibrationIlluminant2: int
    BestQualityScale: int
    RawDataUniqueID: int
    OriginalRawFileName: int
    OriginalRawFileData: int
    ActiveArea: int
    MaskedAreas: int
    AsShotICCProfile: int
    AsShotPreProfileMatrix: int
    CurrentICCProfile: int
    CurrentPreProfileMatrix: int
    ColorimetricReference: int
    CameraCalibrationSignature: int
    ProfileCalibrationSignature: int
    AsShotProfileName: int
    NoiseReductionApplied: int
    ProfileName: int
    ProfileHueSatMapDims: int
    ProfileHueSatMapData1: int
    ProfileHueSatMapData2: int
    ProfileToneCurve: int
    ProfileEmbedPolicy: int
    ProfileCopyright: int
    ForwardMatrix1: int
    ForwardMatrix2: int
    PreviewApplicationName: int
    PreviewApplicationVersion: int
    PreviewSettingsName: int
    PreviewSettingsDigest: int
    PreviewColorSpace: int
    PreviewDateTime: int
    RawImageDigest: int
    OriginalRawFileDigest: int
    SubTileBlockSize: int
    RowInterleaveFactor: int
    ProfileLookTableDims: int
    ProfileLookTableData: int
    OpcodeList1: int
    OpcodeList2: int
    OpcodeList3: int
    NoiseProfile: int

TAGS: Mapping[int, str]

class GPS(IntEnum):
    GPSVersionID: int
    GPSLatitudeRef: int
    GPSLatitude: int
    GPSLongitudeRef: int
    GPSLongitude: int
    GPSAltitudeRef: int
    GPSAltitude: int
    GPSTimeStamp: int
    GPSSatellites: int
    GPSStatus: int
    GPSMeasureMode: int
    GPSDOP: int
    GPSSpeedRef: int
    GPSSpeed: int
    GPSTrackRef: int
    GPSTrack: int
    GPSImgDirectionRef: int
    GPSImgDirection: int
    GPSMapDatum: int
    GPSDestLatitudeRef: int
    GPSDestLatitude: int
    GPSDestLongitudeRef: int
    GPSDestLongitude: int
    GPSDestBearingRef: int
    GPSDestBearing: int
    GPSDestDistanceRef: int
    GPSDestDistance: int
    GPSProcessingMethod: int
    GPSAreaInformation: int
    GPSDateStamp: int
    GPSDifferential: int
    GPSHPositioningError: int

GPSTAGS: Mapping[int, str]

class Interop(IntEnum):
    InteropIndex: int
    InteropVersion: int
    RelatedImageFileFormat: int
    RelatedImageWidth: int
    RleatedImageHeight: int

class IFD(IntEnum):
    Exif: int
    GPSInfo: int
    Makernote: int
    Interop: int
    IFD1: int

class LightSource(IntEnum):
    Unknown: int
    Daylight: int
    Fluorescent: int
    Tungsten: int
    Flash: int
    Fine: int
    Cloudy: int
    Shade: int
    DaylightFluorescent: int
    DayWhiteFluorescent: int
    CoolWhiteFluorescent: int
    WhiteFluorescent: int
    StandardLightA: int
    StandardLightB: int
    StandardLightC: int
    D55: int
    D65: int
    D75: int
    D50: int
    ISO: int
    Other: int



from collections.abc import Mapping, Sequence
from typing import Any

from django.http.request import HttpRequest
from django.template.exceptions import TemplateDoesNotExist as TemplateDoesNotExist  # noqa: F401
from django.utils.safestring import SafeString

from .backends.base import _EngineTemplate

def get_template(template_name: str, using: str | None = ...) -> _EngineTemplate: ...
def select_template(template_name_list: Sequence[str] | str, using: str | None = ...) -> Any: ...
def render_to_string(
    template_name: Sequence[str] | str,
    context: Mapping[str, Any] | None = ...,
    request: HttpRequest | None = ...,
    using: str | None = ...,
) -> SafeString: ...



from typing import Any

from django.template.defaulttags import TemplateLiteral
from typing_extensions import TypeAlias

_Token: TypeAlias = list[int] | int | str

class TokenBase:
    id: Any
    value: Any
    first: Any
    second: Any
    def nud(self, parser: Any) -> None: ...
    def led(self, left: Any, parser: Any) -> None: ...
    def display(self) -> Any: ...

def infix(bp: Any, func: Any) -> type[TokenBase]: ...
def prefix(bp: Any, func: Any) -> type[TokenBase]: ...

OPERATORS: Any

class Literal(TokenBase):
    id: str
    lbp: int
    value: _Token | None
    def __init__(self, value: _Token | None) -> None: ...
    def display(self) -> str: ...
    def eval(self, context: dict[Any, Any]) -> _Token | None: ...

class EndToken(TokenBase):
    lbp: int
    def nud(self, parser: Any) -> None: ...

class IfParser:
    error_class: Any
    tokens: Any
    pos: int
    current_token: Any
    def __init__(self, tokens: list[_Token | None]) -> None: ...
    def translate_token(self, token: _Token | None) -> Literal: ...
    def next_token(self) -> Literal: ...
    def parse(self) -> TemplateLiteral: ...
    def expression(self, rbp: int = ...) -> Literal: ...
    def create_var(self, value: _Token | None) -> Literal: ...



from collections.abc import Callable, Sequence
from typing import Any, TypeVar, Union, overload  # noqa: Y037

from django.contrib.admin import ModelAdmin
from django.contrib.admin.sites import AdminSite
from django.db.models import Combinable, QuerySet
from django.db.models.base import Model
from django.db.models.expressions import BaseExpression
from django.http import HttpRequest, HttpResponseBase
from django.utils.functional import _StrOrPromise
from typing_extensions import TypeAlias

_Model = TypeVar("_Model", bound=Model)
_ModelAdmin = TypeVar("_ModelAdmin", bound=ModelAdmin)
_Request = TypeVar("_Request", bound=HttpRequest)
_QuerySet = TypeVar("_QuerySet", bound=QuerySet)
# This is deliberately different from _DisplayT defined in contrib.admin.options
_DisplayCallable: TypeAlias = Union[Callable[[_ModelAdmin, _Model], Any], Callable[[_Model], Any]]  # noqa: Y037
_DisplayCallableT = TypeVar("_DisplayCallableT", bound=_DisplayCallable)
_ActionReturn = TypeVar("_ActionReturn", bound=HttpResponseBase | None)

@overload
def action(
    function: Callable[[_ModelAdmin, _Request, _QuerySet], _ActionReturn],
    permissions: Sequence[str] | None = ...,
    description: _StrOrPromise | None = ...,
) -> Callable[[_ModelAdmin, _Request, _QuerySet], _ActionReturn]: ...
@overload
def action(
    *,
    permissions: Sequence[str] | None = ...,
    description: _StrOrPromise | None = ...,
) -> Callable[
    [Callable[[_ModelAdmin, _Request, _QuerySet], _ActionReturn]],
    Callable[[_ModelAdmin, _Request, _QuerySet], _ActionReturn],
]: ...
@overload
def display(
    function: _DisplayCallableT,
    boolean: bool | None = ...,
    ordering: str | Combinable | BaseExpression | None = ...,
    description: _StrOrPromise | None = ...,
    empty_value: str | None = ...,
) -> _DisplayCallableT: ...
@overload
def display(
    *,
    boolean: bool | None = ...,
    ordering: str | Combinable | BaseExpression | None = ...,
    description: _StrOrPromise | None = ...,
    empty_value: str | None = ...,
) -> Callable[[_DisplayCallableT], _DisplayCallableT]: ...
def register(
    *models: type[Model], site: AdminSite | None = ...
) -> Callable[[type[_ModelAdmin]], type[_ModelAdmin]]: ...



from collections.abc import Iterable

from rest_framework.parsers import BaseParser
from rest_framework.renderers import BaseRenderer
from rest_framework.request import Request
from rest_framework.settings import api_settings

class BaseContentNegotiation:
    def select_parser(self, request: Request, parsers: Iterable[BaseParser]) -> BaseParser | None: ...
    def select_renderer(self, request: Request, renderers: Iterable[BaseRenderer], format_suffix: str | None = ...): ...

class DefaultContentNegotiation(BaseContentNegotiation):
    settings = api_settings
    def filter_renderers(self, renderers: Iterable[BaseRenderer], format: str) -> list[BaseRenderer]: ...
    def get_accept_list(self, request: Request) -> list[str]: ...



from collections.abc import Sequence
from typing import Any

from django.apps.config import AppConfig
from django.core.checks.messages import CheckMessage, Error, Warning
from django.urls import _AnyURL

def check_url_config(app_configs: Sequence[AppConfig] | None, **kwargs: Any) -> Sequence[CheckMessage]: ...
def check_resolver(resolver: _AnyURL) -> Sequence[CheckMessage]: ...
def check_url_namespaces_unique(app_configs: Sequence[AppConfig] | None, **kwargs: Any) -> Sequence[Warning]: ...
def get_warning_for_invalid_pattern(pattern: Any) -> Sequence[Error]: ...
def check_url_settings(app_configs: Sequence[AppConfig] | None, **kwargs: Any) -> Sequence[Error]: ...
def E006(name: str) -> Error: ...



from _typeshed import Incomplete
from collections.abc import MutableMapping
from numbers import Rational
from types import TracebackType
from typing import Any, ClassVar
from typing_extensions import Final, Literal

from ._imaging import _PixelAccessor
from .ImageFile import ImageFile

# These are meant to be overridable
READ_LIBTIFF: bool
WRITE_LIBTIFF: bool
IFD_LEGACY_API: bool
STRIP_SIZE: int

II: Final = b"II"
MM: Final = b"MM"
IMAGEWIDTH: Final = 256
IMAGELENGTH: Final = 257
BITSPERSAMPLE: Final = 258
COMPRESSION: Final = 259
PHOTOMETRIC_INTERPRETATION: Final = 262
FILLORDER: Final = 266
IMAGEDESCRIPTION: Final = 270
STRIPOFFSETS: Final = 273
SAMPLESPERPIXEL: Final = 277
ROWSPERSTRIP: Final = 278
STRIPBYTECOUNTS: Final = 279
X_RESOLUTION: Final = 282
Y_RESOLUTION: Final = 283
PLANAR_CONFIGURATION: Final = 284
RESOLUTION_UNIT: Final = 296
TRANSFERFUNCTION: Final = 301
SOFTWARE: Final = 305
DATE_TIME: Final = 306
ARTIST: Final = 315
PREDICTOR: Final = 317
COLORMAP: Final = 320
TILEWIDTH: Final = 322
TILELENGTH: Final = 323
TILEOFFSETS: Final = 324
TILEBYTECOUNTS: Final = 325
SUBIFD: Final = 330
EXTRASAMPLES: Final = 338
SAMPLEFORMAT: Final = 339
JPEGTABLES: Final = 347
YCBCRSUBSAMPLING: Final = 530
REFERENCEBLACKWHITE: Final = 532
COPYRIGHT: Final = 33432
IPTC_NAA_CHUNK: Final = 33723
PHOTOSHOP_CHUNK: Final = 34377
ICCPROFILE: Final = 34675
EXIFIFD: Final = 34665
XMP: Final = 700
JPEGQUALITY: Final = 65537
IMAGEJ_META_DATA_BYTE_COUNTS: Final = 50838
IMAGEJ_META_DATA: Final = 50839
COMPRESSION_INFO: Any
COMPRESSION_INFO_REV: Any
OPEN_INFO: Any
MAX_SAMPLESPERPIXEL: Final = 6
PREFIXES: Any

class IFDRational(Rational):
    def __init__(self, value, denominator: int = 1) -> None: ...
    @property
    def numerator(a): ...
    @property
    def denominator(a): ...
    def limit_rational(self, max_denominator): ...
    def __hash__(self) -> int: ...
    def __eq__(self, other): ...
    __add__: Any
    __radd__: Any
    __sub__: Any
    __rsub__: Any
    __mul__: Any
    __rmul__: Any
    __truediv__: Any
    __rtruediv__: Any
    __floordiv__: Any
    __rfloordiv__: Any
    __mod__: Any
    __rmod__: Any
    __pow__: Any
    __rpow__: Any
    __pos__: Any
    __neg__: Any
    __abs__: Any
    __trunc__: Any
    __lt__: Any
    __gt__: Any
    __le__: Any
    __ge__: Any
    __bool__: Any
    __ceil__: Any
    __floor__: Any
    __round__: Any

class ImageFileDirectory_v2(MutableMapping[int, Any]):
    group: int | None
    tagtype: dict[int, int]
    def __init__(
        self, ifh: bytes = b"II*\x00\x00\x00\x00\x00", prefix: bytes | None = None, group: int | None = None
    ) -> None: ...
    @property
    def prefix(self) -> bytes: ...
    @property
    def offset(self) -> int | None: ...
    @property
    def legacy_api(self) -> bool: ...
    def reset(self) -> None: ...
    def named(self): ...
    def __len__(self) -> int: ...
    def __getitem__(self, tag): ...
    def __contains__(self, tag): ...
    def __setitem__(self, tag, value) -> None: ...
    def __delitem__(self, tag) -> None: ...
    def __iter__(self): ...
    def load_byte(self, data, legacy_api: bool = True): ...
    def write_byte(self, data): ...
    def load_string(self, data, legacy_api: bool = True): ...
    def write_string(self, value: int | str | bytes) -> bytes: ...
    def load_rational(self, data, legacy_api: bool = True): ...
    def write_rational(self, *values): ...
    def load_undefined(self, data, legacy_api: bool = True): ...
    def write_undefined(self, value): ...
    def load_signed_rational(self, data, legacy_api: bool = True): ...
    def write_signed_rational(self, *values): ...
    def load(self, fp) -> None: ...
    def tobytes(self, offset: int = 0): ...
    def save(self, fp): ...
    load_double: Incomplete
    load_float: Incomplete
    load_long: Incomplete
    load_long8: Incomplete
    load_short: Incomplete
    load_signed_byte: Incomplete
    load_signed_long: Incomplete
    load_signed_short: Incomplete
    write_double: Incomplete
    write_float: Incomplete
    write_long: Incomplete
    write_long8: Incomplete
    write_short: Incomplete
    write_signed_byte: Incomplete
    write_signed_long: Incomplete
    write_signed_short: Incomplete

class ImageFileDirectory_v1(ImageFileDirectory_v2):
    def __init__(self, *args, **kwargs) -> None: ...
    @property
    def tags(self): ...
    @property
    def tagdata(self): ...
    tagtype: dict[int, int]
    @classmethod
    def from_v2(cls, original): ...
    def to_v2(self): ...
    def __contains__(self, tag): ...
    def __len__(self) -> int: ...
    def __iter__(self): ...
    def __setitem__(self, tag, value) -> None: ...
    def __getitem__(self, tag): ...

ImageFileDirectory = ImageFileDirectory_v1

class TiffImageFile(ImageFile):
    format: ClassVar[Literal["TIFF", "MIC"]]
    format_description: ClassVar[str]
    tag_v2: Any
    tag: Any
    def __init__(self, fp: Incomplete | None = None, filename: Incomplete | None = None) -> None: ...
    @property
    def n_frames(self): ...
    im: Any
    def seek(self, frame) -> None: ...
    def tell(self): ...
    def getxmp(self): ...
    def get_photoshop_blocks(self): ...
    def load(self) -> _PixelAccessor: ...
    def load_end(self) -> None: ...

SAVE_INFO: Any

class AppendingTiffWriter:
    fieldSizes: Any
    Tags: Any
    f: Any
    close_fp: bool
    name: Any
    beginning: Any
    def __init__(self, fn, new: bool = False) -> None: ...
    whereToWriteNewIFDOffset: Any
    offsetOfNewPage: int
    IIMM: Any
    isFirst: bool
    def setup(self) -> None: ...
    def finalize(self) -> None: ...
    def newFrame(self) -> None: ...
    def __enter__(self): ...
    def __exit__(
        self, exc_type: type[BaseException] | None, exc_value: BaseException | None, traceback: TracebackType | None
    ) -> Literal[False]: ...
    def tell(self): ...
    def seek(self, offset, whence=0): ...
    def goToEnd(self) -> None: ...
    endian: Any
    longFmt: Any
    shortFmt: Any
    tagFormat: Any
    def setEndian(self, endian) -> None: ...
    def skipIFDs(self) -> None: ...
    def write(self, data): ...
    def readShort(self): ...
    def readLong(self): ...
    def rewriteLastShortToLong(self, value) -> None: ...
    def rewriteLastShort(self, value) -> None: ...
    def rewriteLastLong(self, value) -> None: ...
    def writeShort(self, value) -> None: ...
    def writeLong(self, value) -> None: ...
    def close(self) -> None: ...
    def fixIFD(self) -> None: ...
    def fixOffsets(self, count, isShort: bool = False, isLong: bool = False) -> None: ...



import array
import threading
import weakref
from queue import Queue as Queue
from typing import Any, Callable, Iterable, List, Mapping, Optional, Sequence

JoinableQueue = Queue
Barrier = threading.Barrier
BoundedSemaphore = threading.BoundedSemaphore
Condition = threading.Condition
Event = threading.Event
Lock = threading.Lock
RLock = threading.RLock
Semaphore = threading.Semaphore

class DummyProcess(threading.Thread):
    _children: weakref.WeakKeyDictionary[Any, Any]
    _parent: threading.Thread
    _pid: None
    _start_called: int
    exitcode: Optional[int]
    def __init__(
        self,
        group: Any = ...,
        target: Optional[Callable[..., Any]] = ...,
        name: Optional[str] = ...,
        args: Iterable[Any] = ...,
        kwargs: Mapping[str, Any] = ...,
    ) -> None: ...

Process = DummyProcess

class Namespace:
    def __init__(self, **kwds: Any) -> None: ...
    def __getattr__(self, __name: str) -> Any: ...
    def __setattr__(self, __name: str, __value: Any) -> None: ...

class Value:
    _typecode: Any
    _value: Any
    value: Any
    def __init__(self, typecode: Any, value: Any, lock: Any = ...) -> None: ...

def Array(typecode: Any, sequence: Sequence[Any], lock: Any = ...) -> array.array[Any]: ...
def Manager() -> Any: ...
def Pool(
    processes: Optional[int] = ..., initializer: Optional[Callable[..., Any]] = ..., initargs: Iterable[Any] = ...
) -> Any: ...
def active_children() -> List[Any]: ...
def current_process() -> threading.Thread: ...
def freeze_support() -> None: ...
def shutdown() -> None: ...



from collections.abc import Callable, Collection, Iterable, Mapping, Sequence, Sized
from typing import Any, TypeVar, overload

from django.template.base import FilterExpression, Origin, Parser, Token
from django.template.context import Context
from django.utils.safestring import SafeString

from .base import Node, Template

class InvalidTemplateLibrary(Exception): ...

_C = TypeVar("_C", bound=Callable[..., Any])

class Library:
    filters: dict[str, Callable]
    tags: dict[str, Callable]
    def __init__(self) -> None: ...
    @overload
    def tag(self, name: _C) -> _C: ...
    @overload
    def tag(self, name: str, compile_function: _C) -> _C: ...
    @overload
    def tag(self, name: str | None = ..., compile_function: None = ...) -> Callable[[_C], _C]: ...
    def tag_function(self, func: _C) -> _C: ...
    @overload
    def filter(self, name: _C, filter_func: None = ..., **flags: Any) -> _C: ...
    @overload
    def filter(self, name: str | None, filter_func: _C, **flags: Any) -> _C: ...
    @overload
    def filter(self, name: str | None = ..., filter_func: None = ..., **flags: Any) -> Callable[[_C], _C]: ...
    @overload
    def simple_tag(self, func: _C) -> _C: ...
    @overload
    def simple_tag(self, takes_context: bool | None = ..., name: str | None = ...) -> Callable[[_C], _C]: ...
    def inclusion_tag(
        self,
        filename: Template | str,
        func: Callable | None = ...,
        takes_context: bool | None = ...,
        name: str | None = ...,
    ) -> Callable[[_C], _C]: ...

class TagHelperNode(Node):
    func: Any
    takes_context: Any
    args: Any
    kwargs: Any
    def __init__(
        self,
        func: Callable,
        takes_context: bool | None,
        args: list[FilterExpression],
        kwargs: dict[str, FilterExpression],
    ) -> None: ...
    def get_resolved_arguments(self, context: Context) -> tuple[list[int], dict[str, SafeString | int]]: ...

class SimpleNode(TagHelperNode):
    args: list[FilterExpression]
    func: Callable
    kwargs: dict[str, FilterExpression]
    origin: Origin
    takes_context: bool | None
    token: Token
    target_var: str | None
    def __init__(
        self,
        func: Callable,
        takes_context: bool | None,
        args: list[FilterExpression],
        kwargs: dict[str, FilterExpression],
        target_var: str | None,
    ) -> None: ...

class InclusionNode(TagHelperNode):
    args: list[FilterExpression]
    func: Callable
    kwargs: dict[str, FilterExpression]
    origin: Origin
    takes_context: bool | None
    token: Token
    filename: Template | str
    def __init__(
        self,
        func: Callable,
        takes_context: bool | None,
        args: list[FilterExpression],
        kwargs: dict[str, FilterExpression],
        filename: Template | str | None,
    ) -> None: ...

def parse_bits(
    parser: Parser,
    bits: Iterable[str],
    params: Sequence[str],
    varargs: str | None,
    varkw: str | None,
    defaults: Sized | None,
    kwonly: Collection[str],
    kwonly_defaults: Mapping[str, int] | None,
    takes_context: bool | None,
    name: str,
) -> tuple[list[FilterExpression], dict[str, FilterExpression]]: ...
def import_library(name: str) -> Library: ...



from collections.abc import Callable, Sequence
from typing import Any

from django.urls.resolvers import ResolverMatch
from typing_extensions import Literal

def resolve(path: str, urlconf: str | None = ...) -> ResolverMatch: ...
def reverse(
    viewname: Callable | str | None,
    urlconf: str | None = ...,
    args: Sequence[Any] | None = ...,
    kwargs: dict[str, Any] | None = ...,
    current_app: str | None = ...,
) -> str: ...

reverse_lazy: Any

def clear_url_caches() -> None: ...
def set_script_prefix(prefix: str) -> None: ...
def get_script_prefix() -> str: ...
def clear_script_prefix() -> None: ...
def set_urlconf(urlconf_name: str | None) -> None: ...
def get_urlconf(default: str | None = ...) -> str | None: ...
def is_valid_path(path: str, urlconf: str | None = ...) -> Literal[False] | ResolverMatch: ...
def translate_url(url: str, lang_code: str) -> str: ...



import abc
import builtins
import codecs
import sys
from _typeshed import FileDescriptorOrPath, ReadableBuffer, WriteableBuffer
from collections.abc import Callable, Iterable, Iterator
from os import _Opener
from types import TracebackType
from typing import IO, Any, BinaryIO, TextIO
from typing_extensions import Literal, Self

__all__ = [
    "BlockingIOError",
    "open",
    "IOBase",
    "RawIOBase",
    "FileIO",
    "BytesIO",
    "StringIO",
    "BufferedIOBase",
    "BufferedReader",
    "BufferedWriter",
    "BufferedRWPair",
    "BufferedRandom",
    "TextIOBase",
    "TextIOWrapper",
    "UnsupportedOperation",
    "SEEK_SET",
    "SEEK_CUR",
    "SEEK_END",
]

if sys.version_info >= (3, 8):
    __all__ += ["open_code"]

DEFAULT_BUFFER_SIZE: Literal[8192]

SEEK_SET: Literal[0]
SEEK_CUR: Literal[1]
SEEK_END: Literal[2]

open = builtins.open

if sys.version_info >= (3, 8):
    def open_code(path: str) -> IO[bytes]: ...

BlockingIOError = builtins.BlockingIOError

class UnsupportedOperation(OSError, ValueError): ...

class IOBase(metaclass=abc.ABCMeta):
    def __iter__(self) -> Iterator[bytes]: ...
    def __next__(self) -> bytes: ...
    def __enter__(self) -> Self: ...
    def __exit__(
        self, exc_type: type[BaseException] | None, exc_val: BaseException | None, exc_tb: TracebackType | None
    ) -> None: ...
    def close(self) -> None: ...
    def fileno(self) -> int: ...
    def flush(self) -> None: ...
    def isatty(self) -> bool: ...
    def readable(self) -> bool: ...
    read: Callable[..., Any]
    def readlines(self, __hint: int = -1) -> list[bytes]: ...
    def seek(self, __offset: int, __whence: int = ...) -> int: ...
    def seekable(self) -> bool: ...
    def tell(self) -> int: ...
    def truncate(self, __size: int | None = ...) -> int: ...
    def writable(self) -> bool: ...
    write: Callable[..., Any]
    def writelines(self, __lines: Iterable[ReadableBuffer]) -> None: ...
    def readline(self, __size: int | None = -1) -> bytes: ...
    def __del__(self) -> None: ...
    @property
    def closed(self) -> bool: ...
    def _checkClosed(self, msg: str | None = ...) -> None: ...  # undocumented

class RawIOBase(IOBase):
    def readall(self) -> bytes: ...
    def readinto(self, __buffer: WriteableBuffer) -> int | None: ...
    def write(self, __b: ReadableBuffer) -> int | None: ...
    def read(self, __size: int = -1) -> bytes | None: ...

class BufferedIOBase(IOBase):
    raw: RawIOBase  # This is not part of the BufferedIOBase API and may not exist on some implementations.
    def detach(self) -> RawIOBase: ...
    def readinto(self, __buffer: WriteableBuffer) -> int: ...
    def write(self, __buffer: ReadableBuffer) -> int: ...
    def readinto1(self, __buffer: WriteableBuffer) -> int: ...
    def read(self, __size: int | None = ...) -> bytes: ...
    def read1(self, __size: int = ...) -> bytes: ...

class FileIO(RawIOBase, BinaryIO):  # type: ignore[misc]  # incompatible definitions of writelines in the base classes
    mode: str
    name: FileDescriptorOrPath  # type: ignore[assignment]
    def __init__(
        self, file: FileDescriptorOrPath, mode: str = ..., closefd: bool = ..., opener: _Opener | None = ...
    ) -> None: ...
    @property
    def closefd(self) -> bool: ...
    def write(self, __b: ReadableBuffer) -> int: ...
    def read(self, __size: int = -1) -> bytes: ...
    def __enter__(self) -> Self: ...

class BytesIO(BufferedIOBase, BinaryIO):  # type: ignore[misc]  # incompatible definitions of methods in the base classes
    def __init__(self, initial_bytes: ReadableBuffer = ...) -> None: ...
    # BytesIO does not contain a "name" field. This workaround is necessary
    # to allow BytesIO sub-classes to add this field, as it is defined
    # as a read-only property on IO[].
    name: Any
    def __enter__(self) -> Self: ...
    def getvalue(self) -> bytes: ...
    def getbuffer(self) -> memoryview: ...
    def read1(self, __size: int | None = -1) -> bytes: ...

class BufferedReader(BufferedIOBase, BinaryIO):  # type: ignore[misc]  # incompatible definitions of methods in the base classes
    def __enter__(self) -> Self: ...
    def __init__(self, raw: RawIOBase, buffer_size: int = ...) -> None: ...
    def peek(self, __size: int = 0) -> bytes: ...

class BufferedWriter(BufferedIOBase, BinaryIO):  # type: ignore[misc]  # incompatible definitions of writelines in the base classes
    def __enter__(self) -> Self: ...
    def __init__(self, raw: RawIOBase, buffer_size: int = ...) -> None: ...
    def write(self, __buffer: ReadableBuffer) -> int: ...

class BufferedRandom(BufferedReader, BufferedWriter):  # type: ignore[misc]  # incompatible definitions of methods in the base classes
    def __enter__(self) -> Self: ...
    def seek(self, __target: int, __whence: int = 0) -> int: ...  # stubtest needs this

class BufferedRWPair(BufferedIOBase):
    def __init__(self, reader: RawIOBase, writer: RawIOBase, buffer_size: int = ...) -> None: ...
    def peek(self, __size: int = ...) -> bytes: ...

class TextIOBase(IOBase):
    encoding: str
    errors: str | None
    newlines: str | tuple[str, ...] | None
    def __iter__(self) -> Iterator[str]: ...  # type: ignore[override]
    def __next__(self) -> str: ...  # type: ignore[override]
    def detach(self) -> BinaryIO: ...
    def write(self, __s: str) -> int: ...
    def writelines(self, __lines: Iterable[str]) -> None: ...  # type: ignore[override]
    def readline(self, __size: int = ...) -> str: ...  # type: ignore[override]
    def readlines(self, __hint: int = -1) -> list[str]: ...  # type: ignore[override]
    def read(self, __size: int | None = ...) -> str: ...

class TextIOWrapper(TextIOBase, TextIO):  # type: ignore[misc]  # incompatible definitions of write in the base classes
    def __init__(
        self,
        buffer: IO[bytes],
        encoding: str | None = ...,
        errors: str | None = ...,
        newline: str | None = ...,
        line_buffering: bool = ...,
        write_through: bool = ...,
    ) -> None: ...
    @property
    def buffer(self) -> BinaryIO: ...
    @property
    def closed(self) -> bool: ...
    @property
    def line_buffering(self) -> bool: ...
    @property
    def write_through(self) -> bool: ...
    def reconfigure(
        self,
        *,
        encoding: str | None = None,
        errors: str | None = None,
        newline: str | None = None,
        line_buffering: bool | None = None,
        write_through: bool | None = None,
    ) -> None: ...
    # These are inherited from TextIOBase, but must exist in the stub to satisfy mypy.
    def __enter__(self) -> Self: ...
    def __iter__(self) -> Iterator[str]: ...  # type: ignore[override]
    def __next__(self) -> str: ...  # type: ignore[override]
    def writelines(self, __lines: Iterable[str]) -> None: ...  # type: ignore[override]
    def readline(self, __size: int = -1) -> str: ...  # type: ignore[override]
    def readlines(self, __hint: int = -1) -> list[str]: ...  # type: ignore[override]
    def seek(self, __cookie: int, __whence: int = 0) -> int: ...  # stubtest needs this

class StringIO(TextIOWrapper):
    def __init__(self, initial_value: str | None = ..., newline: str | None = ...) -> None: ...
    # StringIO does not contain a "name" field. This workaround is necessary
    # to allow StringIO sub-classes to add this field, as it is defined
    # as a read-only property on IO[].
    name: Any
    def getvalue(self) -> str: ...

class IncrementalNewlineDecoder(codecs.IncrementalDecoder):
    def __init__(self, decoder: codecs.IncrementalDecoder | None, translate: bool, errors: str = ...) -> None: ...
    def decode(self, input: ReadableBuffer | str, final: bool = False) -> str: ...
    @property
    def newlines(self) -> str | tuple[str, ...] | None: ...
    def setstate(self, __state: tuple[bytes, int]) -> None: ...



from typing import Any

from django.db import models
from django.db.models.expressions import Func

class CumeDist(Func):
    output_field: models.FloatField

class DenseRank(Func):
    output_field: models.IntegerField

class FirstValue(Func): ...

class LagLeadFunction(Func):
    def __init__(self, expression: str | None, offset: int = ..., default: int | None = ..., **extra: Any) -> None: ...

class Lag(LagLeadFunction): ...
class LastValue(Func): ...
class Lead(LagLeadFunction): ...

class NthValue(Func):
    def __init__(self, expression: str | None, nth: int = ..., **extra: Any) -> None: ...

class Ntile(Func):
    def __init__(self, num_buckets: int = ..., **extra: Any) -> None: ...
    output_field: models.IntegerField

class PercentRank(Func):
    output_field: models.FloatField

class Rank(Func):
    output_field: models.IntegerField

class RowNumber(Func):
    output_field: models.IntegerField



from typing import Any

from .domreg import getDOMImplementation as getDOMImplementation, registerDOMImplementation as registerDOMImplementation

class Node:
    ELEMENT_NODE: int
    ATTRIBUTE_NODE: int
    TEXT_NODE: int
    CDATA_SECTION_NODE: int
    ENTITY_REFERENCE_NODE: int
    ENTITY_NODE: int
    PROCESSING_INSTRUCTION_NODE: int
    COMMENT_NODE: int
    DOCUMENT_NODE: int
    DOCUMENT_TYPE_NODE: int
    DOCUMENT_FRAGMENT_NODE: int
    NOTATION_NODE: int

# ExceptionCode
INDEX_SIZE_ERR: int
DOMSTRING_SIZE_ERR: int
HIERARCHY_REQUEST_ERR: int
WRONG_DOCUMENT_ERR: int
INVALID_CHARACTER_ERR: int
NO_DATA_ALLOWED_ERR: int
NO_MODIFICATION_ALLOWED_ERR: int
NOT_FOUND_ERR: int
NOT_SUPPORTED_ERR: int
INUSE_ATTRIBUTE_ERR: int
INVALID_STATE_ERR: int
SYNTAX_ERR: int
INVALID_MODIFICATION_ERR: int
NAMESPACE_ERR: int
INVALID_ACCESS_ERR: int
VALIDATION_ERR: int

class DOMException(Exception):
    code: int
    def __init__(self, *args: Any, **kw: Any) -> None: ...
    def _get_code(self) -> int: ...

class IndexSizeErr(DOMException): ...
class DomstringSizeErr(DOMException): ...
class HierarchyRequestErr(DOMException): ...
class WrongDocumentErr(DOMException): ...
class InvalidCharacterErr(DOMException): ...
class NoDataAllowedErr(DOMException): ...
class NoModificationAllowedErr(DOMException): ...
class NotFoundErr(DOMException): ...
class NotSupportedErr(DOMException): ...
class InuseAttributeErr(DOMException): ...
class InvalidStateErr(DOMException): ...
class SyntaxErr(DOMException): ...
class InvalidModificationErr(DOMException): ...
class NamespaceErr(DOMException): ...
class InvalidAccessErr(DOMException): ...
class ValidationErr(DOMException): ...

class UserDataHandler:
    NODE_CLONED: int
    NODE_IMPORTED: int
    NODE_DELETED: int
    NODE_RENAMED: int

XML_NAMESPACE: str
XMLNS_NAMESPACE: str
XHTML_NAMESPACE: str
EMPTY_NAMESPACE: None
EMPTY_PREFIX: None



import datetime
from collections.abc import AsyncIterator, Collection, Iterable, Iterator, MutableMapping, Reversible, Sequence, Sized
from typing import Any, Generic, NamedTuple, TypeVar, overload

from _typeshed import Self
from django.db.models import Manager
from django.db.models.base import Model
from django.db.models.expressions import Combinable
from django.db.models.sql.query import Query, RawQuery
from typing_extensions import TypeAlias

_T = TypeVar("_T", bound=Model, covariant=True)
_Row = TypeVar("_Row", covariant=True)
_QS = TypeVar("_QS", bound=_QuerySet)
_TupleT = TypeVar("_TupleT", bound=tuple[Any, ...], covariant=True)

MAX_GET_RESULTS: int
REPR_OUTPUT_SIZE: int

class BaseIterable(Generic[_Row]):
    queryset: QuerySet[Model]
    chunked_fetch: bool
    chunk_size: int
    def __init__(self, queryset: QuerySet[Model], chunked_fetch: bool = ..., chunk_size: int = ...) -> None: ...

class ModelIterable(Generic[_T], BaseIterable[_T]):
    def __iter__(self) -> Iterator[_T]: ...

class RawModelIterable(BaseIterable[dict[str, Any]]):
    def __iter__(self) -> Iterator[dict[str, Any]]: ...

class ValuesIterable(BaseIterable[dict[str, Any]]):
    def __iter__(self) -> Iterator[dict[str, Any]]: ...

class ValuesListIterable(BaseIterable[_TupleT]):
    def __iter__(self) -> Iterator[_TupleT]: ...

class NamedValuesListIterable(ValuesListIterable[NamedTuple]):
    def __iter__(self) -> Iterator[NamedTuple]: ...

class FlatValuesListIterable(BaseIterable[_Row]):
    def __iter__(self) -> Iterator[_Row]: ...

class _QuerySet(Generic[_T, _Row], Collection[_Row], Reversible[_Row], Sized):
    model: type[_T]
    query: Query
    _iterable_class: type[BaseIterable]
    def __init__(
        self,
        model: type[Model] | None = ...,
        query: Query | None = ...,
        using: str | None = ...,
        hints: dict[str, Model] | None = ...,
    ) -> None: ...
    @classmethod
    def as_manager(cls) -> Manager[_T]: ...
    def __len__(self) -> int: ...
    def __bool__(self) -> bool: ...
    def __class_getitem__(cls: type[_QS], item: type[_T]) -> type[_QS]: ...
    def __getstate__(self) -> dict[str, Any]: ...
    # Technically, the other QuerySet must be of the same type _T, but _T is covariant
    def __and__(self: Self, other: _QuerySet[_T, _Row]) -> Self: ...
    def __or__(self: Self, other: _QuerySet[_T, _Row]) -> Self: ...
    # IMPORTANT: When updating any of the following methods' signatures, please ALSO modify
    # the corresponding method in BaseManager.
    def iterator(self, chunk_size: int = ...) -> Iterator[_Row]: ...
    def aggregate(self, *args: Any, **kwargs: Any) -> dict[str, Any]: ...
    async def aaggregate(self, *args: Any, **kwargs: Any) -> dict[str, Any]: ...
    def get(self, *args: Any, **kwargs: Any) -> _Row: ...
    async def aget(self, *args: Any, **kwargs: Any) -> _Row: ...
    def create(self, **kwargs: Any) -> _T: ...
    async def acreate(self, **kwargs: Any) -> _T: ...
    def bulk_create(
        self,
        objs: Iterable[_T],
        batch_size: int | None = ...,
        ignore_conflicts: bool = ...,
        update_conflicts: bool = ...,
        update_fields: Collection[str] | None = ...,
        unique_fields: Collection[str] | None = ...,
    ) -> list[_T]: ...
    async def abulk_create(
        self,
        objs: Iterable[_T],
        batch_size: int | None = ...,
        ignore_conflicts: bool = ...,
        update_conflicts: bool = ...,
        update_fields: Collection[str] | None = ...,
        unique_fields: Collection[str] | None = ...,
    ) -> list[_T]: ...
    def bulk_update(self, objs: Iterable[_T], fields: Iterable[str], batch_size: int | None = ...) -> int: ...
    async def abulk_update(self, objs: Iterable[_T], fields: Iterable[str], batch_size: int | None = ...) -> int: ...
    def get_or_create(self, defaults: MutableMapping[str, Any] | None = ..., **kwargs: Any) -> tuple[_T, bool]: ...
    async def aget_or_create(
        self, defaults: MutableMapping[str, Any] | None = ..., **kwargs: Any
    ) -> tuple[_T, bool]: ...
    def update_or_create(self, defaults: MutableMapping[str, Any] | None = ..., **kwargs: Any) -> tuple[_T, bool]: ...
    async def aupdate_or_create(
        self, defaults: MutableMapping[str, Any] | None = ..., **kwargs: Any
    ) -> tuple[_T, bool]: ...
    def earliest(self, *fields: Any, field_name: Any | None = ...) -> _Row: ...
    async def aearliest(self, *fields: Any, field_name: Any | None = ...) -> _Row: ...
    def latest(self, *fields: Any, field_name: Any | None = ...) -> _Row: ...
    async def alatest(self, *fields: Any, field_name: Any | None = ...) -> _Row: ...
    def first(self) -> _Row | None: ...
    async def afirst(self) -> _Row | None: ...
    def last(self) -> _Row | None: ...
    async def alast(self) -> _Row | None: ...
    def in_bulk(self, id_list: Iterable[Any] = ..., *, field_name: str = ...) -> dict[Any, _T]: ...
    async def ain_bulk(self, id_list: Iterable[Any] = ..., *, field_name: str = ...) -> dict[Any, _T]: ...
    def delete(self) -> tuple[int, dict[str, int]]: ...
    async def adelete(self) -> tuple[int, dict[str, int]]: ...
    def update(self, **kwargs: Any) -> int: ...
    async def aupdate(self, **kwargs: Any) -> int: ...
    def exists(self) -> bool: ...
    async def aexists(self) -> bool: ...
    def explain(self, *, format: Any | None = ..., **options: Any) -> str: ...
    async def aexplain(self, *, format: Any | None = ..., **options: Any) -> str: ...
    def contains(self, obj: Model) -> bool: ...
    async def acontains(self, obj: Model) -> bool: ...
    def raw(
        self,
        raw_query: str,
        params: Any = ...,
        translations: dict[str, str] | None = ...,
        using: str | None = ...,
    ) -> RawQuerySet: ...
    # The type of values may be overridden to be more specific in the mypy plugin, depending on the fields param
    def values(self, *fields: str | Combinable, **expressions: Any) -> _QuerySet[_T, dict[str, Any]]: ...
    # The type of values_list may be overridden to be more specific in the mypy plugin, depending on the fields param
    def values_list(self, *fields: str | Combinable, flat: bool = ..., named: bool = ...) -> _QuerySet[_T, Any]: ...
    def dates(self, field_name: str, kind: str, order: str = ...) -> _QuerySet[_T, datetime.date]: ...
    def datetimes(
        self, field_name: str, kind: str, order: str = ..., tzinfo: datetime.tzinfo | None = ...
    ) -> _QuerySet[_T, datetime.datetime]: ...
    def none(self: Self) -> Self: ...
    def all(self: Self) -> Self: ...
    def filter(self: Self, *args: Any, **kwargs: Any) -> Self: ...
    def exclude(self: Self, *args: Any, **kwargs: Any) -> Self: ...
    def complex_filter(self: Self, filter_obj: Any) -> Self: ...
    def count(self) -> int: ...
    async def acount(self) -> int: ...
    def union(self: Self, *other_qs: Any, all: bool = ...) -> Self: ...
    def intersection(self: Self, *other_qs: Any) -> Self: ...
    def difference(self: Self, *other_qs: Any) -> Self: ...
    def select_for_update(
        self: Self, nowait: bool = ..., skip_locked: bool = ..., of: Sequence[str] = ..., no_key: bool = ...
    ) -> Self: ...
    def select_related(self: Self, *fields: Any) -> Self: ...
    def prefetch_related(self: Self, *lookups: Any) -> Self: ...
    def annotate(self: Self, *args: Any, **kwargs: Any) -> Self: ...
    def alias(self: Self, *args: Any, **kwargs: Any) -> Self: ...
    def order_by(self: Self, *field_names: Any) -> Self: ...
    def distinct(self: Self, *field_names: Any) -> Self: ...
    # extra() return type won't be supported any time soon
    def extra(
        self,
        select: dict[str, Any] | None = ...,
        where: Sequence[str] | None = ...,
        params: Sequence[Any] | None = ...,
        tables: Sequence[str] | None = ...,
        order_by: Sequence[str] | None = ...,
        select_params: Sequence[Any] | None = ...,
    ) -> _QuerySet[Any, Any]: ...
    def reverse(self: Self) -> Self: ...
    def defer(self: Self, *fields: Any) -> Self: ...
    def only(self: Self, *fields: Any) -> Self: ...
    def using(self: Self, alias: str | None) -> Self: ...
    @property
    def ordered(self) -> bool: ...
    @property
    def db(self) -> str: ...
    def resolve_expression(self, *args: Any, **kwargs: Any) -> Any: ...
    def __iter__(self) -> Iterator[_Row]: ...
    def __aiter__(self) -> AsyncIterator[_Row]: ...
    def __contains__(self, x: object) -> bool: ...
    @overload
    def __getitem__(self, i: int) -> _Row: ...
    @overload
    def __getitem__(self: Self, s: slice) -> Self: ...
    def __reversed__(self) -> Iterator[_Row]: ...

class RawQuerySet(Iterable[_T], Sized):
    query: RawQuery
    def __init__(
        self,
        raw_query: RawQuery | str,
        model: type[Model] | None = ...,
        query: Query | None = ...,
        params: tuple[Any] = ...,
        translations: dict[str, str] | None = ...,
        using: str = ...,
        hints: dict[str, Model] | None = ...,
    ) -> None: ...
    def __len__(self) -> int: ...
    def __iter__(self) -> Iterator[_T]: ...
    def __bool__(self) -> bool: ...
    @overload
    def __getitem__(self, k: int) -> _T: ...
    @overload
    def __getitem__(self, k: str) -> Any: ...
    @overload
    def __getitem__(self, k: slice) -> RawQuerySet[_T]: ...
    @property
    def columns(self) -> list[str]: ...
    @property
    def db(self) -> str: ...
    def iterator(self) -> Iterator[_T]: ...
    async def aiterator(self) -> Iterator[_T]: ...
    @property
    def model_fields(self) -> dict[str, str]: ...
    def prefetch_related(self, *lookups: Any) -> RawQuerySet[_T]: ...
    def resolve_model_init_order(self) -> tuple[list[str], list[int], list[tuple[str, int]]]: ...
    def using(self, alias: str | None) -> RawQuerySet[_T]: ...

_QuerySetAny: TypeAlias = _QuerySet  # noqa: Y047

QuerySet: TypeAlias = _QuerySet[_T, _T]

class Prefetch:
    prefetch_through: str
    prefetch_to: str
    queryset: QuerySet | None
    to_attr: str | None
    def __init__(self, lookup: str, queryset: QuerySet | None = ..., to_attr: str | None = ...) -> None: ...
    def __getstate__(self) -> dict[str, Any]: ...
    def add_prefix(self, prefix: str) -> None: ...
    def get_current_prefetch_to(self, level: int) -> str: ...
    def get_current_to_attr(self, level: int) -> tuple[str, str]: ...
    def get_current_queryset(self, level: int) -> QuerySet | None: ...

def prefetch_related_objects(model_instances: Iterable[_T], *related_lookups: str | Prefetch) -> None: ...
def get_prefetcher(instance: Model, through_attr: str, to_attr: str) -> tuple[Any, Any, bool, bool]: ...

class InstanceCheckMeta(type): ...
class EmptyQuerySet(metaclass=InstanceCheckMeta): ...



from rest_framework.permissions import BasePermission


class IsAuthenticated(BasePermission):
    """
    Allows access only to authenticated users.
    """

    def has_permission(self, request, view):
        return bool(request.user)

    def has_object_permission(self, request, view, obj):
        """
        Return `True` if permission is granted, `False` otherwise.
        """
        return True



from typing import Any

def manager_repr(value: Any): ...
def smart_repr(value: Any): ...
def field_repr(field: Any, force_many: bool = ...): ...
def serializer_repr(serializer: Any, indent: Any, force_many: Any | None = ...): ...
def list_repr(serializer: Any, indent: Any): ...



from django.contrib.gis.geos.prototypes.coordseq import create_cs as create_cs
from django.contrib.gis.geos.prototypes.coordseq import cs_clone as cs_clone
from django.contrib.gis.geos.prototypes.coordseq import cs_getdims as cs_getdims
from django.contrib.gis.geos.prototypes.coordseq import cs_getordinate as cs_getordinate
from django.contrib.gis.geos.prototypes.coordseq import cs_getsize as cs_getsize
from django.contrib.gis.geos.prototypes.coordseq import cs_getx as cs_getx
from django.contrib.gis.geos.prototypes.coordseq import cs_gety as cs_gety
from django.contrib.gis.geos.prototypes.coordseq import cs_getz as cs_getz
from django.contrib.gis.geos.prototypes.coordseq import cs_is_ccw as cs_is_ccw
from django.contrib.gis.geos.prototypes.coordseq import cs_setordinate as cs_setordinate
from django.contrib.gis.geos.prototypes.coordseq import cs_setx as cs_setx
from django.contrib.gis.geos.prototypes.coordseq import cs_sety as cs_sety
from django.contrib.gis.geos.prototypes.coordseq import cs_setz as cs_setz
from django.contrib.gis.geos.prototypes.coordseq import get_cs as get_cs
from django.contrib.gis.geos.prototypes.geom import create_collection as create_collection
from django.contrib.gis.geos.prototypes.geom import create_empty_polygon as create_empty_polygon
from django.contrib.gis.geos.prototypes.geom import create_linearring as create_linearring
from django.contrib.gis.geos.prototypes.geom import create_linestring as create_linestring
from django.contrib.gis.geos.prototypes.geom import create_point as create_point
from django.contrib.gis.geos.prototypes.geom import create_polygon as create_polygon
from django.contrib.gis.geos.prototypes.geom import destroy_geom as destroy_geom
from django.contrib.gis.geos.prototypes.geom import geom_clone as geom_clone
from django.contrib.gis.geos.prototypes.geom import geos_get_srid as geos_get_srid
from django.contrib.gis.geos.prototypes.geom import geos_normalize as geos_normalize
from django.contrib.gis.geos.prototypes.geom import geos_set_srid as geos_set_srid
from django.contrib.gis.geos.prototypes.geom import geos_type as geos_type
from django.contrib.gis.geos.prototypes.geom import geos_typeid as geos_typeid
from django.contrib.gis.geos.prototypes.geom import get_dims as get_dims
from django.contrib.gis.geos.prototypes.geom import get_extring as get_extring
from django.contrib.gis.geos.prototypes.geom import get_geomn as get_geomn
from django.contrib.gis.geos.prototypes.geom import get_intring as get_intring
from django.contrib.gis.geos.prototypes.geom import get_nrings as get_nrings
from django.contrib.gis.geos.prototypes.geom import get_num_coords as get_num_coords
from django.contrib.gis.geos.prototypes.geom import get_num_geoms as get_num_geoms
from django.contrib.gis.geos.prototypes.predicates import geos_contains as geos_contains
from django.contrib.gis.geos.prototypes.predicates import geos_covers as geos_covers
from django.contrib.gis.geos.prototypes.predicates import geos_crosses as geos_crosses
from django.contrib.gis.geos.prototypes.predicates import geos_disjoint as geos_disjoint
from django.contrib.gis.geos.prototypes.predicates import geos_equals as geos_equals
from django.contrib.gis.geos.prototypes.predicates import geos_equalsexact as geos_equalsexact
from django.contrib.gis.geos.prototypes.predicates import geos_hasz as geos_hasz
from django.contrib.gis.geos.prototypes.predicates import geos_intersects as geos_intersects
from django.contrib.gis.geos.prototypes.predicates import geos_isclosed as geos_isclosed
from django.contrib.gis.geos.prototypes.predicates import geos_isempty as geos_isempty
from django.contrib.gis.geos.prototypes.predicates import geos_isring as geos_isring
from django.contrib.gis.geos.prototypes.predicates import geos_issimple as geos_issimple
from django.contrib.gis.geos.prototypes.predicates import geos_isvalid as geos_isvalid
from django.contrib.gis.geos.prototypes.predicates import geos_overlaps as geos_overlaps
from django.contrib.gis.geos.prototypes.predicates import geos_relatepattern as geos_relatepattern
from django.contrib.gis.geos.prototypes.predicates import geos_touches as geos_touches
from django.contrib.gis.geos.prototypes.predicates import geos_within as geos_within



from typing import IO

from django.core.files import File
from django.utils._os import _PathCompatible

class ImageFile(File[bytes]):
    @property
    def width(self) -> int: ...
    @property
    def height(self) -> int: ...

def get_image_dimensions(
    file_or_path: _PathCompatible | IO[bytes], close: bool = ...
) -> tuple[int | None, int | None]: ...



from collections.abc import Iterator
from typing import Any

from django.http.request import HttpRequest
from django.http.response import HttpResponseBase

LEVEL_TAGS: Any

class Message:
    level: int
    message: str
    extra_tags: str
    def __init__(self, level: int, message: str, extra_tags: str | None = ...) -> None: ...
    @property
    def tags(self) -> str: ...
    @property
    def level_tag(self) -> str: ...

class BaseStorage:
    request: HttpRequest
    used: bool
    added_new: bool
    def __init__(self, request: HttpRequest, *args: Any, **kwargs: Any) -> None: ...
    def __len__(self) -> int: ...
    def __iter__(self) -> Iterator[Message]: ...
    def __contains__(self, item: Any) -> bool: ...
    def update(self, response: HttpResponseBase) -> list[Message] | None: ...
    def add(self, level: int, message: str, extra_tags: str | None = ...) -> None: ...
    level: Any



from collections.abc import Callable
from typing import Any, TypeVar, overload

from _typeshed import Self
from typing_extensions import TypeAlias

_SD = TypeVar("_SD", bound=SafeData)

class SafeData:
    def __html__(self: Self) -> Self: ...

class SafeString(str, SafeData):
    @overload
    def __add__(self, rhs: SafeString) -> SafeString: ...
    @overload
    def __add__(self, rhs: str) -> str: ...

SafeText: TypeAlias = SafeString

_C = TypeVar("_C", bound=Callable)

@overload
def mark_safe(s: _SD) -> _SD: ...
@overload
def mark_safe(s: _C) -> _C: ...
@overload
def mark_safe(s: Any) -> SafeString: ...



from collections.abc import Iterable, Iterator

from django.core.files.storage import Storage

def matches_patterns(path: str, patterns: Iterable[str]) -> bool: ...
def get_files(storage: Storage, ignore_patterns: Iterable[str] | None = ..., location: str = ...) -> Iterator[str]: ...
def check_settings(base_url: str | None = ...) -> None: ...



from celery.utils.dispatch import Signal

__all__ = [
    "before_task_publish",
    "after_task_publish",
    "task_prerun",
    "task_postrun",
    "task_success",
    "task_retry",
    "task_failure",
    "task_revoked",
    "celeryd_init",
    "celeryd_after_setup",
    "worker_init",
    "worker_process_init",
    "worker_ready",
    "worker_shutdown",
    "worker_shutting_down",
    "setup_logging",
    "after_setup_logger",
    "after_setup_task_logger",
    "beat_init",
    "beat_embedded_init",
    "heartbeat_sent",
    "eventlet_pool_started",
    "eventlet_pool_preshutdown",
    "eventlet_pool_postshutdown",
    "eventlet_pool_apply",
]

before_task_publish: Signal
after_task_publish: Signal
task_received: Signal
task_prerun: Signal
task_postrun: Signal
task_success: Signal
task_retry: Signal
task_failure: Signal
task_revoked: Signal
task_rejected: Signal
task_unknown: Signal
#: Deprecated, use after_task_publish instead.
task_sent: Signal

celeryd_init: Signal
celeryd_after_setup: Signal

import_modules: Signal
worker_init: Signal
worker_process_init: Signal
worker_process_shutdown: Signal
worker_ready: Signal
worker_shutdown: Signal
worker_shutting_down: Signal
heartbeat_sent: Signal

setup_logging: Signal
after_setup_logger: Signal
after_setup_task_logger: Signal

beat_init: Signal
beat_embedded_init: Signal

eventlet_pool_started: Signal
eventlet_pool_preshutdown: Signal
eventlet_pool_postshutdown: Signal
eventlet_pool_apply: Signal

user_preload_options: Signal



from argparse import ArgumentParser, HelpFormatter, Namespace
from collections.abc import Callable, Iterable, Sequence
from io import TextIOBase
from typing import Any, TextIO

from django.apps.config import AppConfig
from django.core.management.color import Style
from django.utils.datastructures import _ListOrTuple
from typing_extensions import Literal

class CommandError(Exception):
    def __init__(self, *args: Any, returncode: int = ..., **kwargs: Any) -> None: ...

class SystemCheckError(CommandError): ...

class CommandParser(ArgumentParser):
    missing_args_message: str | None
    called_from_command_line: bool | None
    def __init__(
        self, *, missing_args_message: str | None = ..., called_from_command_line: bool | None = ..., **kwargs: Any
    ) -> None: ...
    def error(self, message: str) -> Any: ...

def handle_default_options(options: Namespace) -> None: ...
def no_translations(handle_func: Callable) -> Callable: ...

class DjangoHelpFormatter(HelpFormatter):
    show_last: set[str]
    def add_usage(self, usage: str | None, actions: Iterable[Any], *args: Any, **kwargs: Any) -> Any: ...
    def add_arguments(self, actions: Any) -> Any: ...

class OutputWrapper(TextIOBase):
    @property
    def style_func(self) -> Callable[[str], str]: ...
    @style_func.setter
    def style_func(self, style_func: Callable[[str], str] | None) -> None: ...
    ending: str
    def __init__(self, out: TextIO, ending: str = ...) -> None: ...
    def __getattr__(self, name: str) -> Callable: ...
    def flush(self) -> None: ...
    def isatty(self) -> bool: ...
    def write(  # type: ignore[override]
        self, msg: str = ..., style_func: Callable[[str], str] | None = ..., ending: str | None = ...
    ) -> None: ...

class BaseCommand:
    help: str
    output_transaction: bool
    requires_migrations_checks: bool
    requires_system_checks: _ListOrTuple[str] | Literal["__all__"]
    base_stealth_options: tuple[str, ...]
    stealth_options: tuple[str, ...]
    stdout: OutputWrapper
    stderr: OutputWrapper
    style: Style
    def __init__(
        self,
        stdout: TextIO | None = ...,
        stderr: TextIO | None = ...,
        no_color: bool = ...,
        force_color: bool = ...,
    ) -> None: ...
    def get_version(self) -> str: ...
    def create_parser(self, prog_name: str, subcommand: str, **kwargs: Any) -> CommandParser: ...
    def add_arguments(self, parser: CommandParser) -> None: ...
    def print_help(self, prog_name: str, subcommand: str) -> None: ...
    def run_from_argv(self, argv: list[str]) -> None: ...
    def execute(self, *args: Any, **options: Any) -> str | None: ...
    def check(
        self,
        app_configs: Sequence[AppConfig] | None = ...,
        tags: Sequence[str] | None = ...,
        display_num_errors: bool = ...,
        include_deployment_checks: bool = ...,
        fail_level: int = ...,
        databases: Sequence[str] | None = ...,
    ) -> None: ...
    def check_migrations(self) -> None: ...
    def handle(self, *args: Any, **options: Any) -> str | None: ...

class AppCommand(BaseCommand):
    missing_args_message: str
    def add_arguments(self, parser: CommandParser) -> None: ...
    def handle(self, *app_labels: str, **options: Any) -> str | None: ...
    def handle_app_config(self, app_config: Any, **options: Any) -> str | None: ...

class LabelCommand(BaseCommand):
    label: str
    missing_args_message: Any
    def add_arguments(self, parser: CommandParser) -> None: ...
    def handle(self, *labels: str, **options: Any) -> str | None: ...
    def handle_label(self, label: str, **options: Any) -> str | None: ...



from typing import Any

from django.http.request import HttpRequest
from django.http.response import HttpResponse, HttpResponsePermanentRedirect
from django.utils.deprecation import MiddlewareMixin

class SecurityMiddleware(MiddlewareMixin):
    sts_seconds: int
    sts_include_subdomains: bool
    sts_preload: bool
    content_type_nosniff: bool
    xss_filter: bool
    redirect: bool
    redirect_host: str | None
    redirect_exempt: list[Any]
    def process_request(self, request: HttpRequest) -> HttpResponsePermanentRedirect | None: ...
    def process_response(self, request: HttpRequest, response: HttpResponse) -> HttpResponse: ...



# Stubs for django.conf.urls (Python 3.5)
from collections.abc import Callable, Sequence
from typing import Any, overload

from django.http.response import HttpResponse, HttpResponseBase
from django.urls import URLPattern, URLResolver
from django.urls import include as include
from typing_extensions import TypeAlias

handler400: str | Callable[..., HttpResponse]
handler403: str | Callable[..., HttpResponse]
handler404: str | Callable[..., HttpResponse]
handler500: str | Callable[..., HttpResponse]

IncludedURLConf: TypeAlias = tuple[Sequence[URLResolver | URLPattern], str | None, str | None]

# Deprecated
@overload
def url(
    regex: str, view: Callable[..., HttpResponseBase], kwargs: dict[str, Any] | None = ..., name: str | None = ...
) -> URLPattern: ...
@overload
def url(
    regex: str, view: IncludedURLConf, kwargs: dict[str, Any] | None = ..., name: str | None = ...
) -> URLResolver: ...
@overload
def url(
    regex: str,
    view: Sequence[URLResolver | str],
    kwargs: dict[str, Any] | None = ...,
    name: str | None = ...,
) -> URLResolver: ...



from typing import Any, TypeVar

__all__ = ["Error", "copy", "deepcopy"]

_T = TypeVar("_T")

# None in CPython but non-None in Jython
PyStringMap: Any

# Note: memo and _nil are internal kwargs.
def deepcopy(x: _T, memo: dict[int, Any] | None = None, _nil: Any = []) -> _T: ...
def copy(x: _T) -> _T: ...

class Error(Exception): ...

error = Error



import logging
from typing import Type

from django.db.models import Model
from humps.main import decamelize
from nwon_baseline.typings import AnyDict
from rest_framework.serializers import Serializer

from nwon_django_toolbox.settings import NWON_DJANGO_SETTINGS

LOGGER = logging.getLogger(NWON_DJANGO_SETTINGS.logger_name)


def dictionary_is_serialized_instance(
    instance: Model,
    serializer: Type[Serializer],
    response: AnyDict,
    decamelize_response: bool = True,
) -> bool:
    """
    Checks that a dictionary resembles the serialization of a model instance
    with a certain serializer.
    """

    """
    We need to decamelize the dict as well in case there is some nested data in it that
    has camel case keys.
    """
    serialized_dict = serializer(instance=instance).data
    serialized_dict = (
        decamelize(serialized_dict) if decamelize_response else serialized_dict
    )

    compare_dict = decamelize(response) if decamelize_response else response

    is_equal = serialized_dict == compare_dict

    if not is_equal:
        difference = {
            k: compare_dict[k] for k in set(compare_dict) - set(serialized_dict)
        }
        LOGGER.debug(
            f"Dictionary differs from serialized data from {serializer.__name__}: \n {difference}"
        )

    return is_equal


__all__ = ["dictionary_is_serialized_instance"]



from typing import Any

from _typeshed import Self
from django.utils.functional import LazyObject

# explicit dependency on standard settings to make it loaded
from . import global_settings

ENVIRONMENT_VARIABLE: str
PASSWORD_RESET_TIMEOUT_DAYS_DEPRECATED_MSG: str
DEFAULT_HASHING_ALGORITHM_DEPRECATED_MSG: str

# required for plugin to be able to distinguish this specific instance of LazySettings from others
class _DjangoConfLazyObject(LazyObject):
    def __getattr__(self, item: Any) -> Any: ...

class LazySettings(_DjangoConfLazyObject):
    SETTINGS_MODULE: str
    @property
    def configured(self) -> bool: ...
    def configure(self, default_settings: Any = ..., **options: Any) -> None: ...

settings: LazySettings

class Settings:
    SETTINGS_MODULE: str
    def __init__(self, settings_module: str) -> None: ...
    def is_overridden(self, setting: str) -> bool: ...

class UserSettingsHolder:
    SETTINGS_MODULE: None
    def __init__(self, default_settings: Any) -> None: ...
    def __getattr__(self, name: str) -> Any: ...
    def __setattr__(self, name: str, value: Any) -> None: ...
    def __delattr__(self, name: str) -> None: ...
    def is_overridden(self, setting: str) -> bool: ...

class SettingsReference(str):
    def __new__(self: type[Self], value: Any, setting_name: str) -> Self: ...
    def __init__(self, value: str, setting_name: str) -> None: ...



from typing import Any, TypeVar

from django.db.models import Model
from rest_framework.generics import UsesQuerySet
from rest_framework.request import Request
from rest_framework.response import Response
from rest_framework.serializers import BaseSerializer

_MT = TypeVar("_MT", bound=Model)

class CreateModelMixin:
    def create(self, request: Request, *args: Any, **kwargs: Any) -> Response: ...
    def perform_create(self: UsesQuerySet[_MT], serializer: BaseSerializer[_MT]) -> None: ...
    def get_success_headers(self, data: Any) -> dict[str, str]: ...

class ListModelMixin:
    def list(self, request: Request, *args: Any, **kwargs: Any) -> Response: ...

class RetrieveModelMixin:
    def retrieve(self, request: Request, *args: Any, **kwargs: Any) -> Response: ...

class UpdateModelMixin:
    def update(self, request: Request, *args: Any, **kwargs: Any) -> Response: ...
    def perform_update(self: UsesQuerySet[_MT], serializer: BaseSerializer[_MT]) -> None: ...
    def partial_update(self, request: Request, *args: Any, **kwargs: Any) -> Response: ...

class DestroyModelMixin:
    def destroy(self, request: Request, *args: Any, **kwargs: Any) -> Response: ...
    def perform_destroy(self: UsesQuerySet[_MT], instance: _MT) -> None: ...



import json
from typing import Any

from django.db.backends.base.base import BaseDatabaseWrapper
from django.db.models import lookups
from django.db.models.lookups import PostgresOperatorLookup, Transform
from django.db.models.sql.compiler import SQLCompiler
from django.utils.functional import _StrOrPromise

from . import Field
from .mixins import CheckFieldDefaultMixin

class JSONField(CheckFieldDefaultMixin, Field):
    encoder: type[json.JSONEncoder] | None
    decoder: type[json.JSONDecoder] | None
    def __init__(
        self,
        verbose_name: _StrOrPromise | None = ...,
        name: str | None = ...,
        encoder: type[json.JSONEncoder] | None = ...,
        decoder: type[json.JSONDecoder] | None = ...,
        **kwargs: Any
    ) -> None: ...

class DataContains(PostgresOperatorLookup): ...
class ContainedBy(PostgresOperatorLookup): ...

class HasKeyLookup(PostgresOperatorLookup):
    logical_operator: str | None

class HasKey(HasKeyLookup):
    postgres_operator: str

class HasKeys(HasKeyLookup):
    postgres_operator: str
    logical_operator: str

class HasAnyKeys(HasKeys):
    postgres_operator: str
    logical_operator: str

class JSONExact(lookups.Exact): ...

class KeyTransform(Transform):
    key_name: str
    postgres_operator: str
    postgres_nested_operator: str
    def __init__(self, key_name: Any, *args: Any, **kwargs: Any) -> None: ...
    def preprocess_lhs(self, compiler: SQLCompiler, connection: BaseDatabaseWrapper) -> Any: ...

class KeyTextTransform(KeyTransform):
    postgres_operator: str
    postgres_nested_operator: str

class KeyTransformTextLookupMixin:
    def __init__(self, key_transform: Any, *args: Any, **kwargs: Any) -> None: ...

class CaseInsensitiveMixin: ...
class KeyTransformIsNull(lookups.IsNull): ...
class KeyTransformIn(lookups.In): ...
class KeyTransformExact(JSONExact): ...
class KeyTransformIExact(CaseInsensitiveMixin, KeyTransformTextLookupMixin, lookups.IExact): ...
class KeyTransformIContains(CaseInsensitiveMixin, KeyTransformTextLookupMixin, lookups.IContains): ...
class KeyTransformStartsWith(KeyTransformTextLookupMixin, lookups.StartsWith): ...
class KeyTransformIStartsWith(CaseInsensitiveMixin, KeyTransformTextLookupMixin, lookups.IStartsWith): ...
class KeyTransformEndsWith(KeyTransformTextLookupMixin, lookups.EndsWith): ...
class KeyTransformIEndsWith(CaseInsensitiveMixin, KeyTransformTextLookupMixin, lookups.IEndsWith): ...
class KeyTransformRegex(KeyTransformTextLookupMixin, lookups.Regex): ...
class KeyTransformIRegex(CaseInsensitiveMixin, KeyTransformTextLookupMixin, lookups.IRegex): ...
class KeyTransformNumericLookupMixin: ...
class KeyTransformLt(KeyTransformNumericLookupMixin, lookups.LessThan): ...
class KeyTransformLte(KeyTransformNumericLookupMixin, lookups.LessThanOrEqual): ...
class KeyTransformGt(KeyTransformNumericLookupMixin, lookups.GreaterThan): ...
class KeyTransformGte(KeyTransformNumericLookupMixin, lookups.GreaterThanOrEqual): ...

class KeyTransformFactory:
    key_name: Any
    def __init__(self, key_name: Any) -> None: ...
    def __call__(self, *args: Any, **kwargs: Any) -> KeyTransform: ...



from collections.abc import Awaitable, Callable, Iterable, Iterator, Mapping
from io import BytesIO
from json import JSONEncoder
from re import Pattern
from types import TracebackType
from typing import Any, Generic, NoReturn, TypeVar

from django.contrib.auth.base_user import AbstractBaseUser
from django.contrib.sessions.backends.base import SessionBase
from django.core.handlers.asgi import ASGIRequest
from django.core.handlers.base import BaseHandler
from django.core.handlers.wsgi import WSGIRequest
from django.http.cookie import SimpleCookie
from django.http.request import HttpRequest
from django.http.response import HttpResponseBase
from django.template.base import Template
from django.test.utils import ContextList
from django.urls import ResolverMatch
from typing_extensions import TypeAlias

BOUNDARY: str
MULTIPART_CONTENT: str
CONTENT_TYPE_RE: Pattern
JSON_CONTENT_TYPE_RE: Pattern

class RedirectCycleError(Exception):
    last_response: HttpResponseBase
    redirect_chain: list[tuple[str, int]]
    def __init__(self, message: str, last_response: HttpResponseBase) -> None: ...

class FakePayload:
    read_started: bool
    def __init__(self, content: bytes | str | None = ...) -> None: ...
    def __len__(self) -> int: ...
    def read(self, num_bytes: int = ...) -> bytes: ...
    def write(self, content: bytes | str) -> None: ...

_T = TypeVar("_T")

def closing_iterator_wrapper(iterable: Iterable[_T], close: Callable[[], Any]) -> Iterator[_T]: ...
def conditional_content_removal(request: HttpRequest, response: HttpResponseBase) -> HttpResponseBase: ...

class _WSGIResponse(HttpResponseBase):
    wsgi_request: WSGIRequest

class _ASGIResponse(HttpResponseBase):
    asgi_request: ASGIRequest

class ClientHandler(BaseHandler):
    enforce_csrf_checks: bool
    def __init__(self, enforce_csrf_checks: bool = ..., *args: Any, **kwargs: Any) -> None: ...
    def __call__(self, environ: dict[str, Any]) -> _WSGIResponse: ...

class AsyncClientHandler(BaseHandler):
    enforce_csrf_checks: bool
    def __init__(self, enforce_csrf_checks: bool = ..., *args: Any, **kwargs: Any) -> None: ...
    async def __call__(self, scope: dict[str, Any]) -> _ASGIResponse: ...

def encode_multipart(boundary: str, data: dict[str, Any]) -> bytes: ...
def encode_file(boundary: str, key: str, file: Any) -> list[bytes]: ...

_GetDataType: TypeAlias = (
    Mapping[str, str | bytes | int | Iterable[str | bytes | int]]
    | Iterable[tuple[str, str | bytes | int | Iterable[str | bytes | int]]]
    | None
)

class _RequestFactory(Generic[_T]):
    json_encoder: type[JSONEncoder]
    defaults: dict[str, str]
    cookies: SimpleCookie
    errors: BytesIO
    def __init__(self, *, json_encoder: type[JSONEncoder] = ..., **defaults: Any) -> None: ...
    def request(self, **request: Any) -> _T: ...
    def get(self, path: str, data: _GetDataType = ..., secure: bool = ..., **extra: Any) -> _T: ...
    def post(self, path: str, data: Any = ..., content_type: str = ..., secure: bool = ..., **extra: Any) -> _T: ...
    def head(self, path: str, data: Any = ..., secure: bool = ..., **extra: Any) -> _T: ...
    def trace(self, path: str, secure: bool = ..., **extra: Any) -> _T: ...
    def options(
        self, path: str, data: dict[str, str] | str = ..., content_type: str = ..., secure: bool = ..., **extra: Any
    ) -> _T: ...
    def put(self, path: str, data: Any = ..., content_type: str = ..., secure: bool = ..., **extra: Any) -> _T: ...
    def patch(self, path: str, data: Any = ..., content_type: str = ..., secure: bool = ..., **extra: Any) -> _T: ...
    def delete(self, path: str, data: Any = ..., content_type: str = ..., secure: bool = ..., **extra: Any) -> _T: ...
    def generic(
        self, method: str, path: str, data: Any = ..., content_type: str | None = ..., secure: bool = ..., **extra: Any
    ) -> _T: ...

class RequestFactory(_RequestFactory[WSGIRequest]): ...

class _AsyncRequestFactory(_RequestFactory[_T]):
    def request(self, **request: Any) -> _T: ...
    def generic(
        self, method: str, path: str, data: Any = ..., content_type: str | None = ..., secure: bool = ..., **extra: Any
    ) -> _T: ...

class AsyncRequestFactory(_AsyncRequestFactory[ASGIRequest]): ...

# fakes to distinguish WSGIRequest and ASGIRequest
class _MonkeyPatchedWSGIResponse(_WSGIResponse):
    def json(self) -> Any: ...
    request: dict[str, Any]
    client: Client
    templates: list[Template]
    context: ContextList | dict[str, Any]
    content: bytes
    resolver_match: ResolverMatch
    redirect_chain: list[tuple[str, int]]

class _MonkeyPatchedASGIResponse(_ASGIResponse):
    def json(self) -> Any: ...
    request: dict[str, Any]
    client: AsyncClient
    templates: list[Template]
    context: ContextList | dict[str, Any]
    content: bytes
    resolver_match: ResolverMatch
    redirect_chain: list[tuple[str, int]]

class ClientMixin:
    def store_exc_info(self, **kwargs: Any) -> None: ...
    def check_exception(self, response: HttpResponseBase) -> NoReturn: ...
    @property
    def session(self) -> SessionBase: ...
    def login(self, **credentials: Any) -> bool: ...
    def force_login(self, user: AbstractBaseUser, backend: str | None = ...) -> None: ...
    def logout(self) -> None: ...

class Client(ClientMixin, _RequestFactory[_MonkeyPatchedWSGIResponse]):
    handler: ClientHandler
    raise_request_exception: bool
    exc_info: tuple[type[BaseException], BaseException, TracebackType] | None
    def __init__(
        self, enforce_csrf_checks: bool = ..., raise_request_exception: bool = ..., **defaults: Any
    ) -> None: ...
    # Silence type warnings, since this class overrides arguments and return types in an unsafe manner.
    def request(self, **request: Any) -> _MonkeyPatchedWSGIResponse: ...
    def get(  # type: ignore
        self, path: str, data: _GetDataType = ..., follow: bool = ..., secure: bool = ..., **extra: Any
    ) -> _MonkeyPatchedWSGIResponse: ...
    def post(  # type: ignore
        self, path: str, data: Any = ..., content_type: str = ..., follow: bool = ..., secure: bool = ..., **extra: Any
    ) -> _MonkeyPatchedWSGIResponse: ...
    def head(  # type: ignore
        self, path: str, data: Any = ..., follow: bool = ..., secure: bool = ..., **extra: Any
    ) -> _MonkeyPatchedWSGIResponse: ...
    def trace(  # type: ignore
        self, path: str, data: Any = ..., follow: bool = ..., secure: bool = ..., **extra: Any
    ) -> _MonkeyPatchedWSGIResponse: ...
    def put(  # type: ignore
        self, path: str, data: Any = ..., content_type: str = ..., follow: bool = ..., secure: bool = ..., **extra: Any
    ) -> _MonkeyPatchedWSGIResponse: ...
    def patch(  # type: ignore
        self, path: str, data: Any = ..., content_type: str = ..., follow: bool = ..., secure: bool = ..., **extra: Any
    ) -> _MonkeyPatchedWSGIResponse: ...
    def delete(  # type: ignore
        self, path: str, data: Any = ..., content_type: str = ..., follow: bool = ..., secure: bool = ..., **extra: Any
    ) -> _MonkeyPatchedWSGIResponse: ...

class AsyncClient(ClientMixin, _AsyncRequestFactory[Awaitable[_MonkeyPatchedASGIResponse]]):
    handler: AsyncClientHandler
    raise_request_exception: bool
    exc_info: Any
    extra: Any
    def __init__(
        self, enforce_csrf_checks: bool = ..., raise_request_exception: bool = ..., **defaults: Any
    ) -> None: ...
    async def request(self, **request: Any) -> _MonkeyPatchedASGIResponse: ...



import decimal
from collections.abc import Callable, Iterable, Iterator, Mapping
from contextlib import AbstractContextManager, contextmanager
from decimal import Decimal
from io import StringIO
from logging import Logger
from types import TracebackType
from typing import Any, Protocol, TypeVar

from _typeshed import Self
from django.apps.registry import Apps
from django.conf import LazySettings, Settings
from django.core.checks.registry import CheckRegistry
from django.db.backends.base.base import BaseDatabaseWrapper
from django.db.models.lookups import Lookup, Transform
from django.db.models.query_utils import RegisterLookupMixin
from django.test.runner import DiscoverRunner
from django.test.testcases import SimpleTestCase
from typing_extensions import SupportsIndex, TypeAlias

_TestClass: TypeAlias = type[SimpleTestCase]

_DecoratedTest: TypeAlias = Callable | _TestClass
_DT = TypeVar("_DT", bound=_DecoratedTest)
_C = TypeVar("_C", bound=Callable)  # Any callable

TZ_SUPPORT: bool

class Approximate:
    val: decimal.Decimal | float
    places: int
    def __init__(self, val: Decimal | float, places: int = ...) -> None: ...

class ContextList(list[dict[str, Any]]):
    def __getitem__(self, key: str | SupportsIndex | slice) -> Any: ...
    def get(self, key: str, default: Any | None = ...) -> Any: ...
    def __contains__(self, key: object) -> bool: ...
    def keys(self) -> set[str]: ...

class _TestState: ...

def setup_test_environment(debug: bool | None = ...) -> None: ...
def teardown_test_environment() -> None: ...
def get_runner(settings: LazySettings, test_runner_class: str | None = ...) -> type[DiscoverRunner]: ...

class TestContextDecorator:
    attr_name: str | None
    kwarg_name: str | None
    def __init__(self, attr_name: str | None = ..., kwarg_name: str | None = ...) -> None: ...
    def enable(self) -> Any: ...
    def disable(self) -> None: ...
    def __enter__(self) -> Apps | None: ...  # noqa: Y034
    def __exit__(
        self,
        exc_type: type[BaseException] | None,
        exc_value: BaseException | None,
        exc_tb: TracebackType | None,
    ) -> None: ...
    def decorate_class(self, cls: _TestClass) -> _TestClass: ...
    def decorate_callable(self, func: _C) -> _C: ...
    def __call__(self, decorated: _DT) -> _DT: ...

class override_settings(TestContextDecorator):
    options: dict[str, Any]
    def __init__(self, **kwargs: Any) -> None: ...
    wrapped: Settings
    def save_options(self, test_func: _DecoratedTest) -> None: ...
    def decorate_class(self, cls: type) -> type: ...

class modify_settings(override_settings):
    wrapped: Settings
    operations: list[tuple[str, dict[str, list[str] | str]]]
    def __init__(self, *args: Any, **kwargs: Any) -> None: ...
    def save_options(self, test_func: _DecoratedTest) -> None: ...
    options: dict[str, list[tuple[str, str] | str]]

class override_system_checks(TestContextDecorator):
    registry: CheckRegistry
    new_checks: list[Callable]
    deployment_checks: list[Callable] | None
    def __init__(self, new_checks: list[Callable], deployment_checks: list[Callable] | None = ...) -> None: ...
    old_checks: set[Callable]
    old_deployment_checks: set[Callable]

class CaptureQueriesContext:
    connection: BaseDatabaseWrapper
    force_debug_cursor: bool
    initial_queries: int
    final_queries: int | None
    def __init__(self, connection: BaseDatabaseWrapper) -> None: ...
    def __iter__(self) -> Iterator[dict[str, str]]: ...
    def __getitem__(self, index: int) -> dict[str, str]: ...
    def __len__(self) -> int: ...
    @property
    def captured_queries(self) -> list[dict[str, str]]: ...
    def __enter__(self: Self) -> Self: ...
    def __exit__(
        self,
        exc_type: type[BaseException] | None,
        exc_value: BaseException | None,
        exc_tb: TracebackType | None,
    ) -> None: ...

class ignore_warnings(TestContextDecorator):
    ignore_kwargs: dict[str, Any]
    filter_func: Callable
    def __init__(self, **kwargs: Any) -> None: ...
    catch_warnings: AbstractContextManager[list | None]

requires_tz_support: Any

@contextmanager
def isolate_lru_cache(lru_cache_object: Callable) -> Iterator[None]: ...

class override_script_prefix(TestContextDecorator):
    prefix: str
    def __init__(self, prefix: str) -> None: ...
    old_prefix: str

class LoggingCaptureMixin:
    logger: Logger
    old_stream: Any
    logger_output: Any
    def setUp(self) -> None: ...
    def tearDown(self) -> None: ...

class isolate_apps(TestContextDecorator):
    installed_apps: tuple[str]
    def __init__(self, *installed_apps: Any, **kwargs: Any) -> None: ...
    old_apps: Apps

@contextmanager
def extend_sys_path(*paths: str) -> Iterator[None]: ...
@contextmanager
def captured_output(stream_name: str) -> Iterator[StringIO]: ...
def captured_stdin() -> AbstractContextManager: ...
def captured_stdout() -> AbstractContextManager: ...
def captured_stderr() -> AbstractContextManager: ...
@contextmanager
def freeze_time(t: float) -> Iterator[None]: ...
def tag(*tags: str) -> Callable[[_C], _C]: ...

_Signature: TypeAlias = str
_TestDatabase: TypeAlias = tuple[str, list[str]]

class TimeKeeperProtocol(Protocol):
    @contextmanager
    def timed(self, name: Any) -> Iterator[None]: ...
    def print_results(self) -> None: ...

def dependency_ordered(
    test_databases: Iterable[tuple[_Signature, _TestDatabase]], dependencies: Mapping[str, list[str]]
) -> list[tuple[_Signature, _TestDatabase]]: ...
def get_unique_databases_and_mirrors(
    aliases: set[str] | None = ...,
) -> tuple[dict[_Signature, _TestDatabase], dict[str, Any]]: ...
def setup_databases(
    verbosity: int,
    interactive: bool,
    *,
    time_keeper: TimeKeeperProtocol | None = ...,
    keepdb: bool = ...,
    debug_sql: bool = ...,
    parallel: int = ...,
    aliases: Mapping[str, Any] | None = ...,
    **kwargs: Any
) -> list[tuple[BaseDatabaseWrapper, str, bool]]: ...
def teardown_databases(
    old_config: Iterable[tuple[Any, str, bool]], verbosity: int, parallel: int = ..., keepdb: bool = ...
) -> None: ...
def require_jinja2(test_func: _C) -> _C: ...
@contextmanager
def register_lookup(
    field: type[RegisterLookupMixin], *lookups: type[Lookup | Transform], lookup_name: str | None = ...
) -> Iterator[None]: ...



from typing import Iterable, TypeVar, Mapping, Union


StrOrIter = TypeVar('StrOrIter', bound=Union[str, Mapping, list])


def pascalize(str_or_iter: StrOrIter) -> StrOrIter: ...
def camelize(str_or_iter: StrOrIter) -> StrOrIter: ...
def kebabize(str_or_iter: StrOrIter) -> StrOrIter: ...
def decamelize(str_or_iter: StrOrIter) -> StrOrIter: ...
def depascalize(str_or_iter: StrOrIter) -> StrOrIter: ...
def dekebabize(str_or_iter: StrOrIter) -> StrOrIter: ...
def is_camelcase(str_or_iter: StrOrIter) -> bool: ...
def is_pascalcase(str_or_iter: StrOrIter) -> bool: ...
def is_kebabcase(str_or_iter: StrOrIter) -> bool: ...
def is_snakecase(str_or_iter: StrOrIter) -> bool: ...



import ssl
from typing import Any

from .. import exceptions

SSLError = exceptions.SSLError
InsecurePlatformWarning = exceptions.InsecurePlatformWarning
SSLContext = ssl.SSLContext

HAS_SNI: Any
create_default_context: Any
OP_NO_SSLv2: Any
OP_NO_SSLv3: Any
OP_NO_COMPRESSION: Any
DEFAULT_CIPHERS: str

def assert_fingerprint(cert, fingerprint): ...
def resolve_cert_reqs(candidate): ...
def resolve_ssl_version(candidate): ...
def create_urllib3_context(ssl_version=None, cert_reqs=None, options=None, ciphers=None): ...
def ssl_wrap_socket(
    sock,
    keyfile=None,
    certfile=None,
    cert_reqs=None,
    ca_certs=None,
    server_hostname=None,
    ssl_version=None,
    ciphers=None,
    ssl_context=None,
): ...



from collections import OrderedDict
from collections.abc import Callable, Iterable, Mapping, Sequence
from typing import Any, Generic, TypeVar

from _typeshed import Self
from django.db.models import Manager, Model, QuerySet
from rest_framework.fields import Field, Option
from rest_framework.request import Request
from rest_framework.validators import Validator

def method_overridden(method_name: str, klass: type, instance: Model) -> bool: ...

class ObjectValueError(ValueError): ...
class ObjectTypeError(TypeError): ...

class Hyperlink(str):
    def __new__(cls: type[Self], url: str, obj: Any) -> Self: ...
    def __getnewargs__(self): ...
    @property
    def name(self) -> str: ...
    is_hyperlink: bool
    obj: Any

class PKOnlyObject:
    pk: Any
    def __init__(self, pk: Any) -> None: ...

MANY_RELATION_KWARGS: Sequence[str]

_MT = TypeVar("_MT", bound=Model)
_DT = TypeVar("_DT")  # Data Type
_PT = TypeVar("_PT")  # Primitive Type

class RelatedField(Generic[_MT, _DT, _PT], Field[_MT, _DT, _PT, Any]):
    queryset: QuerySet[_MT] | Manager[_MT] | None
    html_cutoff: int | None
    html_cutoff_text: str | None
    def __init__(
        self,
        many: bool = ...,
        allow_empty: bool = ...,
        queryset: QuerySet[_MT] | Manager[_MT] | None = ...,
        html_cutoff: int | None = ...,
        html_cutoff_text: str = ...,
        read_only: bool = ...,
        write_only: bool = ...,
        required: bool = ...,
        default: Any = ...,
        initial: Any = ...,
        source: Callable | str = ...,
        label: str | None = ...,
        help_text: str = ...,
        allow_null: bool = ...,
        validators: Sequence[Validator[_MT]] | None = ...,
        error_messages: dict[str, str] | None = ...,
        style: dict[str, str] | None = ...,
    ): ...
    # mypy doesn't accept the typing below, although its accurate to what this class is doing, hence the ignore
    def __new__(cls, *args: Any, **kwargs: Any) -> RelatedField[_MT, _DT, _PT] | ManyRelatedField: ...  # type: ignore
    @classmethod
    def many_init(cls, *args: Any, **kwargs: Any) -> ManyRelatedField: ...
    def get_queryset(self) -> QuerySet[_MT]: ...
    def use_pk_only_optimization(self) -> bool: ...
    def get_choices(self, cutoff: int | None = ...) -> OrderedDict: ...
    @property
    def choices(self) -> OrderedDict: ...
    @property
    def grouped_choices(self) -> OrderedDict: ...
    def iter_options(self) -> Iterable[Option]: ...
    def get_attribute(self, instance: _MT) -> _PT | None: ...  # type: ignore[override]
    def display_value(self, instance: _MT) -> str: ...

class StringRelatedField(RelatedField[_MT, _MT, str]): ...

class PrimaryKeyRelatedField(RelatedField[_MT, _MT, Any]):
    pk_field: str | None
    def __init__(
        self,
        many: bool = ...,
        allow_empty: bool = ...,
        queryset: QuerySet[_MT] | Manager[_MT] | None = ...,
        html_cutoff: int | None = ...,
        html_cutoff_text: str = ...,
        read_only: bool = ...,
        write_only: bool = ...,
        required: bool = ...,
        default: Any = ...,
        initial: Any = ...,
        source: Callable | str = ...,
        label: str | None = ...,
        help_text: str = ...,
        allow_null: bool = ...,
        validators: Sequence[Validator[_MT]] | None = ...,
        error_messages: dict[str, str] | None = ...,
        style: dict[str, str] | None = ...,
        pk_field: str | Field | None = ...,
    ): ...

class HyperlinkedRelatedField(RelatedField[_MT, str, Hyperlink]):
    reverse: Callable
    lookup_field: str
    lookup_url_kwarg: str
    format: str | None
    view_name: str | None
    def __init__(
        self,
        many: bool = ...,
        allow_empty: bool = ...,
        queryset: QuerySet[_MT] | Manager[_MT] | None = ...,
        html_cutoff: int | None = ...,
        html_cutoff_text: str = ...,
        read_only: bool = ...,
        write_only: bool = ...,
        required: bool = ...,
        default: Any = ...,
        initial: Any = ...,
        source: Callable | str = ...,
        label: str | None = ...,
        help_text: str = ...,
        allow_null: bool = ...,
        validators: Sequence[Validator[_MT]] | None = ...,
        error_messages: dict[str, str] | None = ...,
        style: dict[str, str] | None = ...,
        view_name: str | None = ...,
        lookup_field: str | None = ...,
        lookup_url_kwarg: str | None = ...,
        format: str | None = ...,
    ): ...
    def get_object(self, view_name: str, *view_args: Any, **view_kwargs: Any) -> _MT: ...
    def get_url(self, obj: Model, view_name: str, request: Request, format: str) -> str | None: ...

class HyperlinkedIdentityField(HyperlinkedRelatedField): ...

class SlugRelatedField(RelatedField[_MT, str, str]):
    slug_field: str | None
    def __init__(
        self,
        many: bool = ...,
        allow_empty: bool = ...,
        queryset: QuerySet[_MT] | Manager[_MT] | None = ...,
        html_cutoff: int | None = ...,
        html_cutoff_text: str = ...,
        read_only: bool = ...,
        write_only: bool = ...,
        required: bool = ...,
        default: _DT = ...,
        initial: _MT | Callable[[Any], _MT] = ...,
        source: Callable | str = ...,
        label: str | None = ...,
        help_text: str = ...,
        allow_null: bool = ...,
        validators: Sequence[Validator[_MT]] | None = ...,
        error_messages: dict[str, str] | None = ...,
        style: dict[str, str] | None = ...,
        slug_field: str | None = ...,
    ): ...
    def to_internal_value(self, data: Any) -> _MT: ...
    def to_representation(self, value: _MT) -> str: ...

class ManyRelatedField(Field[Sequence[Any], Sequence[Any], list[Any], Any]):
    default_empty_html: list[object]
    html_cutoff: int | None
    html_cutoff_text: str | None
    child_relation: RelatedField
    allow_empty: bool
    def __init__(
        self,
        read_only: bool = ...,
        write_only: bool = ...,
        required: bool = ...,
        default: Sequence[Any] = ...,
        initial: Sequence[Any] | Callable[[Any], Sequence[Any]] = ...,
        source: Callable | str = ...,
        label: str | None = ...,
        help_text: str | None = ...,
        style: dict[str, str] | None = ...,
        error_messages: dict[str, str] | None = ...,
        validators: Sequence[Validator[Sequence[Any]]] | None = ...,
        allow_null: bool = ...,
        allow_empty: bool = ...,
        child_relation: RelatedField = ...,
    ): ...
    def get_value(self, dictionary: Mapping[Any, Any]) -> list[Any]: ...  # type: ignore[override]
    def get_choices(self, cutoff: int | None = ...) -> OrderedDict: ...
    @property
    def choices(self) -> OrderedDict: ...
    @property
    def grouped_choices(self) -> OrderedDict: ...
    def iter_options(self) -> Iterable[Option]: ...



import sys
from collections.abc import Iterable
from typing import TextIO
from typing_extensions import Self

if sys.version_info >= (3, 8):
    __all__ = ["shlex", "split", "quote", "join"]
else:
    __all__ = ["shlex", "split", "quote"]

def split(s: str, comments: bool = False, posix: bool = True) -> list[str]: ...

if sys.version_info >= (3, 8):
    def join(split_command: Iterable[str]) -> str: ...

def quote(s: str) -> str: ...

class shlex(Iterable[str]):
    commenters: str
    wordchars: str
    whitespace: str
    escape: str
    quotes: str
    escapedquotes: str
    whitespace_split: bool
    infile: str | None
    instream: TextIO
    source: str
    debug: int
    lineno: int
    token: str
    eof: str | None
    @property
    def punctuation_chars(self) -> str: ...
    def __init__(
        self,
        instream: str | TextIO | None = None,
        infile: str | None = None,
        posix: bool = False,
        punctuation_chars: bool | str = False,
    ) -> None: ...
    def get_token(self) -> str | None: ...
    def push_token(self, tok: str) -> None: ...
    def read_token(self) -> str | None: ...
    def sourcehook(self, newfile: str) -> tuple[str, TextIO] | None: ...
    def push_source(self, newstream: str | TextIO, newfile: str | None = None) -> None: ...
    def pop_source(self) -> None: ...
    def error_leader(self, infile: str | None = None, lineno: int | None = None) -> str: ...
    def __iter__(self) -> Self: ...
    def __next__(self) -> str: ...
